@@ -39,9 +39,14 @@ using namespace OpenCTL;
 // LLVM
 #include <llvm/Module.h>
 #include <llvm/Bitcode/ReaderWriter.h>
+#ifdef LLVM_27_OR_28
 #include <llvm/System/Path.h>
-#include <llvm/Module.h>
 #include <llvm/System/DynamicLibrary.h>
+#else
+#include <llvm/Support/Path.h>
+#include <llvm/Support/DynamicLibrary.h>
+#endif
+#include <llvm/Module.h>
 #include <llvm/LLVMContext.h>
 
 struct Module::Private {
