@@ -30,12 +30,12 @@
 #include <llvm/Analysis/LoopPass.h>
 #include <llvm/Analysis/Verifier.h>
 #include <llvm/Target/TargetData.h>
-#include <llvm/ExecutionEngine/ExecutionEngine.h>
 #include <llvm/GlobalVariable.h>
 
 // GTLCore
 #include "GTLCore/Buffer.h"
 #include "GTLCore/BufferImage.h"
+#include "GTLCore/Debug_thread_p.h"
 #include "GTLCore/LLVMBackend/CodeGenerator_p.h"
 #include "GTLCore/LLVMBackend/GenerationContext_p.h"
 #include "GTLCore/PixelDescription.h"
@@ -48,6 +48,7 @@
 #include "GTLCore/Type.h"
 #include "GTLCore/Type_p.h"
 #include "GTLCore/Value.h"
+#include "GTLCore/Value_p.h"
 #include "GTLCore/VariableNG_p.h"
 #include "GTLCore/Utils_p.h"
 
@@ -68,15 +69,16 @@ struct Program::Private {
   {
     GTL_ASSERT(srcPixelDescriptions.size() > 0);
   }
-  llvm::Module* module;
   GTLCore::ModuleData* moduleData;
   void (*func)( const char**, char*, int, GTLCore::ProgressReport*, gtl_uint64);
   std::list<GTLCore::PixelDescription> srcPixelDescriptions;
   GTLCore::PixelDescription dstPixelDescription;
   static int s_id;
-  std::map< GTLCore::String, llvm::GlobalVariable*> varyings;
-  std::map< GTLCore::String, void*> varyingsPtr;
+  std::vector< llvm::GlobalVariable*> varyings;
+  std::vector< void*> varyingsPtr;
+  std::map< GTLCore::String, int > varyingsId;
   std::list<GTLCore::String> varyingsName;
+  TEST_GUARD_VARIABLE
 };
 
 int Program::Private::s_id = 0;
@@ -146,15 +148,16 @@ struct InputGenInfo {
     InputGenInfo& operator=(const InputGenInfo&) { return *this; }
 };
 
-void Program::init(const GTLCore::String& functionName, const Module* module)
+void Program::init(const GTLCore::String& functionName, const Module* _module)
 {
+  TEST_GUARD_WRITE(d)
   d->func = 0;
   d->moduleData = 0;
   // Clone the module
-  if( module->data())
+  if( _module->data())
   {
-    d->module = llvm::CloneModule(module->data()->llvmLinkedModule());
-    d->moduleData = new GTLCore::ModuleData( d->module );
+    llvm::Module* module = llvm::CloneModule(_module->data()->llvmLinkedModule());
+    d->moduleData = new GTLCore::ModuleData( module );
     // Select the function
     std::vector<GTLCore::Parameter> arguments;
     foreach(const GTLCore::PixelDescription& pd, d->srcPixelDescriptions)
@@ -168,10 +171,10 @@ void Program::init(const GTLCore::String& functionName, const Module* module)
     {
       arguments.push_back(GTLCore::Parameter("", type, true, false, GTLCore::Value()));
     }
-    const GTLCore::Function* functionDef = module->function( functionName, arguments );
+    const GTLCore::Function* functionDef = _module->function( functionName, arguments );
     if( not functionDef ) return;
     
-    llvm::Function* function = d->module->getFunction( (const std::string&)GTLCore::Function::Data::symbolName( GTLCore::ScopedName( "", functionName), functionDef->returnType(),functionDef->parameters() ) );
+    llvm::Function* function = module->getFunction( (const std::string&)GTLCore::Function::Data::symbolName( GTLCore::ScopedName( "", functionName), functionDef->returnType(),functionDef->parameters() ) );
     if(function)
     {
       LLVMBackend::CodeGenerator cg( d->moduleData );
@@ -181,13 +184,13 @@ void Program::init(const GTLCore::String& functionName, const Module* module)
       for( std::list<GTLCore::PixelDescription>::iterator it = d->srcPixelDescriptions.begin();
            it != d->srcPixelDescriptions.end(); ++it )
       {
-        InputGenInfo* src = new InputGenInfo(*it, d->module->getContext());
+        InputGenInfo* src = new InputGenInfo(*it, module->getContext());
         srcs.push_back( src);
-        configureBuffer( cg, src->channelsTypes, src->indexes, src->needConversion, src->countChannels, d->module->getContext());
+        configureBuffer( cg, src->channelsTypes, src->indexes, src->needConversion, src->countChannels, module->getContext());
       }
       // Initialise pixel information and Configure output buffers
-      InputGenInfo dst( d->dstPixelDescription, d->module->getContext());
-      configureBuffer( cg, dst.channelsTypes, dst.indexes, dst.needConversion, dst.countChannels, d->module->getContext());
+      InputGenInfo dst( d->dstPixelDescription, module->getContext());
+      configureBuffer( cg, dst.channelsTypes, dst.indexes, dst.needConversion, dst.countChannels, module->getContext());
       
       //---------- Program-Pseudo-code ----------//
       // This just a rough explanation of the function that get generated here.
@@ -211,21 +214,24 @@ void Program::init(const GTLCore::String& functionName, const Module* module)
       //   return;
       // }
       OCTL_DEBUG("Initialize the function type");
-      std::vector<const llvm::Type*> params;
-      params.push_back( llvm::PointerType::get( llvm::PointerType::get( llvm::Type::getInt8Ty(d->module->getContext()), 0 ), 0 ) ); // const char** int8
-      params.push_back( llvm::PointerType::get( llvm::Type::getInt8Ty(d->module->getContext()), 0 )); // char* out8
-      params.push_back( llvm::Type::getInt32Ty(d->module->getContext()) ); // int size
-      params.push_back( GTLCore::Type::Pointer->d->type(d->module->getContext()) ); // ProgressReport* _report
-      params.push_back( llvm::Type::getInt64Ty(d->module->getContext()) );
+      std::list<GTLCore::String> keepVisible;
+      std::vector<llvm::Type*> params;
+      params.push_back( llvm::PointerType::get( llvm::PointerType::get( llvm::Type::getInt8Ty(module->getContext()), 0 ), 0 ) ); // const char** int8
+      params.push_back( llvm::PointerType::get( llvm::Type::getInt8Ty(module->getContext()), 0 )); // char* out8
+      params.push_back( llvm::Type::getInt32Ty(module->getContext()) ); // int size
+      params.push_back( GTLCore::Type::Pointer->d->type(module->getContext()) ); // ProgressReport* _report
+      params.push_back( llvm::Type::getInt64Ty(module->getContext()) );
       // void program(const char* in8, char* out8, int size)
-      llvm::FunctionType* definitionType = llvm::FunctionType::get( llvm::Type::getVoidTy(d->module->getContext()), params, false );
+      llvm::FunctionType* definitionType = llvm::FunctionType::get( llvm::Type::getVoidTy(module->getContext()), params, false );
       int programNb = ++Program::Private::s_id;
-      llvm::Function* func = cg.createFunction( d->module, definitionType, "CTLProgram" + GTLCore::String::number(programNb));
+      GTLCore::String programFunctionName = "CTLProgram" + GTLCore::String::number(programNb);
+      keepVisible.push_back(programFunctionName);
+      llvm::Function* func = cg.createFunction( module, definitionType, programFunctionName);
       // Initialise a generation context
-      LLVMBackend::GenerationContext gc( &cg, &d->module->getContext(), func, 0, d->moduleData, d->module );
+      LLVMBackend::GenerationContext gc( &cg, &module->getContext(), func, 0, d->moduleData, module );
       // {
       OCTL_DEBUG("Initial block");
-      llvm::BasicBlock* initialBlock = llvm::BasicBlock::Create(d->module->getContext());
+      llvm::BasicBlock* initialBlock = llvm::BasicBlock::Create(module->getContext());
       func->getBasicBlockList().push_back( initialBlock );
       // Initialise the buffer, as needed
       OCTL_DEBUG("Initialise buffer");
@@ -255,7 +261,7 @@ void Program::init(const GTLCore::String& functionName, const Module* module)
       
       for(std::size_t k = 0; k < srcs.size(); ++k)
       {
-        srcs[k]->addr = new llvm::LoadInst( llvm::GetElementPtrInst::Create( ins, cg.integerToConstant(d->module->getContext(), gtl_uint32(k)), "", initialBlock ), "", initialBlock );
+        srcs[k]->addr = new llvm::LoadInst( llvm::GetElementPtrInst::Create( ins, cg.integerToConstant(module->getContext(), gtl_uint32(k)), "", initialBlock ), "", initialBlock );
       }
       //   char* out8 = second arg;
       ++arg_it;
@@ -278,20 +284,20 @@ void Program::init(const GTLCore::String& functionName, const Module* module)
       for(std::size_t k = 0; k < srcs.size(); ++k)
       {
         srcs[k]->pos = new GTLCore::VariableNG( GTLCore::Type::Integer32, false, false );
-        srcs[k]->pos->initialise( gc, initialBlock, LLVMBackend::ExpressionResult(cg.integerToConstant(d->module->getContext(), INT32_C(0)), GTLCore::Type::Integer32), std::list<llvm::Value*>());
+        srcs[k]->pos->initialise( gc, initialBlock, LLVMBackend::ExpressionResult(cg.integerToConstant(module->getContext(), INT32_C(0)), GTLCore::Type::Integer32), std::list<llvm::Value*>());
       }
       dst.pos = new GTLCore::VariableNG( GTLCore::Type::Integer32, false, false );
-      dst.pos->initialise( gc, initialBlock, LLVMBackend::ExpressionResult(cg.integerToConstant(d->module->getContext(), INT32_C(0)), GTLCore::Type::Integer32), std::list<llvm::Value*>());
+      dst.pos->initialise( gc, initialBlock, LLVMBackend::ExpressionResult(cg.integerToConstant(module->getContext(), INT32_C(0)), GTLCore::Type::Integer32), std::list<llvm::Value*>());
       
       // i < size
       OCTL_DEBUG("i < size");
-      llvm::BasicBlock* forTestBlock = llvm::BasicBlock::Create(d->module->getContext(), "forTestBlock");
+      llvm::BasicBlock* forTestBlock = llvm::BasicBlock::Create(module->getContext(), "forTestBlock");
       func->getBasicBlockList().push_back( forTestBlock);
       llvm::Value* forTest = cg.createStrictInferiorExpression(forTestBlock, srcs[0]->pos->get( gc, forTestBlock ), srcs[0]->pos->type(), size, GTLCore::Type::Integer32 );
       
       // i += pixelSize
       OCTL_DEBUG("i += pixelSize");
-      llvm::BasicBlock* updateBlock = llvm::BasicBlock::Create(d->module->getContext(), "updateBlock");
+      llvm::BasicBlock* updateBlock = llvm::BasicBlock::Create(module->getContext(), "updateBlock");
       func->getBasicBlockList().push_back( updateBlock);
       for(std::size_t k = 0; k < srcs.size(); ++k)
       {
@@ -301,7 +307,7 @@ void Program::init(const GTLCore::String& functionName, const Module* module)
       
       // Construct the body of the for loop
       OCTL_DEBUG("bodyBlock");
-      llvm::BasicBlock* bodyBlock = llvm::BasicBlock::Create(d->module->getContext(), "bodyBlock");
+      llvm::BasicBlock* bodyBlock = llvm::BasicBlock::Create(module->getContext(), "bodyBlock");
       llvm::BasicBlock* startBodyBlock = bodyBlock;
       func->getBasicBlockList().push_back( bodyBlock);
       
@@ -319,7 +325,7 @@ void Program::init(const GTLCore::String& functionName, const Module* module)
           OCTL_DEBUG("Load the value from the input buffer");
           llvm::Value* convertedIn = new llvm::LoadInst(
                         cg.convertPointerTo( bodyBlock,
-                                  llvm::GetElementPtrInst::Create( srcs[k]->addr, cg.createAdditionExpression( bodyBlock, srcIndex, srcs[k]->pos->type(), srcs[k]->indexes[i], GTLCore::Type::Integer32), "", bodyBlock), srcs[k]->channelsTypes[i]->d->type(d->module->getContext())),
+                                  llvm::GetElementPtrInst::Create( srcs[k]->addr, cg.createAdditionExpression( bodyBlock, srcIndex, srcs[k]->pos->type(), srcs[k]->indexes[i], GTLCore::Type::Integer32), "", bodyBlock), srcs[k]->channelsTypes[i]->d->type(module->getContext())),
                         "", bodyBlock);
           if( srcs[k]->needConversion[i])
           { // if a buffer is needed that means that the value must be converted
@@ -351,13 +357,15 @@ void Program::init(const GTLCore::String& functionName, const Module* module)
         {
           llvm::GlobalVariable* globalVar =
               new llvm::GlobalVariable(
-                    *d->module,
-                    parameters[i].type()->d->type(d->module->getContext()),
+                    *module,
+                    parameters[i].type()->d->type(module->getContext()),
                     false, llvm::GlobalValue::ExternalLinkage,
-                    LLVMBackend::CodeGenerator::valueToConstant( gc, parameters[i].defaultValue() ), "" );
+                    LLVMBackend::CodeGenerator::valueToConstant( gc, parameters[i].defaultValue() ), (const std::string&)parameters[i].name() );
           arguments.push_back( new llvm::LoadInst( globalVar, "", bodyBlock ) );
-          d->varyings[ parameters[i].name() ] = globalVar;
+          d->varyingsId[ parameters[i].name() ] = d->varyings.size();
+          d->varyings.push_back(globalVar);
           d->varyingsName.push_back( parameters[i].name() );
+          keepVisible.push_back(parameters[i].name());
         }
       }
       // Some debug
@@ -369,7 +377,7 @@ void Program::init(const GTLCore::String& functionName, const Module* module)
       }
       OCTL_DEBUG( *function->getFunctionType());
 #endif
-      llvm::CallInst *CallFunc = llvm::CallInst::Create(function, arguments.begin(), arguments.end(), "", bodyBlock);
+      llvm::CallInst *CallFunc = llvm::CallInst::Create(function, arguments, "", bodyBlock);
       CallFunc->setTailCall(false);
       // If there was buffering, save to output
       OCTL_DEBUG("If there was buffering, save to output");
@@ -379,7 +387,7 @@ void Program::init(const GTLCore::String& functionName, const Module* module)
         llvm::BasicBlock* ifBlock = llvm::BasicBlock::Create(gc.llvmContext());
         func->getBasicBlockList().push_back( ifBlock );
         
-        const llvm::Type* channelType = dst.channelsTypes[i]->d->type(d->module->getContext());
+        llvm::Type* channelType = dst.channelsTypes[i]->d->type(module->getContext());
         llvm::Value* pointer = cg.convertPointerTo( ifBlock, llvm::GetElementPtrInst::Create( out, cg.createAdditionExpression( ifBlock, dstIndex, GTLCore::Type::Integer32, dst.indexes[i], GTLCore::Type::Integer32), "", ifBlock), channelType);
         llvm::Value* result = buffer[i]->get( gc, ifBlock );
         if( dst.needConversion[i])
@@ -422,23 +430,24 @@ void Program::init(const GTLCore::String& functionName, const Module* module)
       OCTL_DEBUG("Put the for loop together");
       // for(int i = 0; i < size; ++i)
       OCTL_DEBUG("for(int i = 0; i < size; ++i)");
-      llvm::BasicBlock* finBlock = llvm::BasicBlock::Create(d->module->getContext(), "finBlock");
+      llvm::BasicBlock* finBlock = llvm::BasicBlock::Create(module->getContext(), "finBlock");
       func->getBasicBlockList().push_back( finBlock);
       cg.createForStatement(initialBlock, forTestBlock, forTestBlock, forTest, GTLCore::Type::Boolean, updateBlock, startBodyBlock, bodyBlock, finBlock);
       // return;
       OCTL_DEBUG("return;");
-      llvm::ReturnInst::Create(d->module->getContext(), finBlock);
-      OCTL_DEBUG(*d->module);
+      llvm::ReturnInst::Create(module->getContext(), finBlock);
+      OCTL_DEBUG(*module);
       // Optimize
       OCTL_DEBUG("Optimize");
-      
+      module = 0;
+      d->moduleData->doLink();
+      d->moduleData->hideAllSymbolsBut(keepVisible);
       // Register module in the VM
-      GTLCore::VirtualMachine::instance()->registerModule( d->module );
-      GTLCore::VirtualMachine::instance()->executionEngine()->clearAllGlobalMappings();
+      GTLCore::VirtualMachine::instance()->registerModule( d->moduleData->llvmLinkedModule());
       d->func = ( void(*)(const char**, char*,int,GTLCore::ProgressReport*, gtl_uint64)) GTLCore::VirtualMachine::instance()->getPointerToFunction( func );
-      for( std::map< GTLCore::String, llvm::GlobalVariable*>::iterator it = d->varyings.begin(); it != d->varyings.end(); ++it)
+      for( std::vector< llvm::GlobalVariable*>::iterator it = d->varyings.begin(); it != d->varyings.end(); ++it)
       {
-          d->varyingsPtr[ it->first ] = GTLCore::VirtualMachine::instance()->getGlobalVariablePointer( it->second );
+          d->varyingsPtr.push_back(GTLCore::VirtualMachine::instance()->getGlobalVariablePointer( *it ) );
       }
 //       OCTL_DEBUG( ((void*)d->func) << " ==== " << func << "  " << func->isDeclaration() << *func);
       // Cleanup
@@ -450,22 +459,24 @@ void Program::init(const GTLCore::String& functionName, const Module* module)
       delete[] buffer;
     } else {
       OCTL_DEBUG("Function: " << functionName << " not found in module");
-      delete d->module;
-      d->module = 0;
+      delete module;
+      module = 0;
     }
   } else {
     OCTL_DEBUG("No module was supplied");
-    d->module = 0;
   }
 }
 
 Program::~Program()
 {
-  if(d->module)
   {
-    GTLCore::VirtualMachine::instance()->unregisterModule( d->module);
+    TEST_GUARD_WRITE(d)
+    if(d->moduleData and d->moduleData->llvmLinkedModule())
+    {
+      GTLCore::VirtualMachine::instance()->unregisterModule( d->moduleData->llvmLinkedModule());
+    }
+    delete d->moduleData;
   }
-  delete d->moduleData;
   delete d;
 }
 
@@ -476,6 +487,7 @@ bool Program::isInitialised() const
 
 void Program::apply(const GTLCore::Buffer& input, GTLCore::Buffer& output, GTLCore::ProgressReport* _report, const GTLCore::ChannelsFlags& flags ) const
 {
+  TEST_GUARD_READ(d)
   GTL_ASSERT(d->srcPixelDescriptions.size() == 1);
   OCTL_ASSERT( (input.size() / (d->srcPixelDescriptions.begin()->bitsSize() / 8) ) == (output.size() / (d->dstPixelDescription.bitsSize() / 8) ) );
   OCTL_ASSERT( d->func );
@@ -490,6 +502,7 @@ void Program::apply(const GTLCore::Buffer& input, GTLCore::Buffer& output, GTLCo
 
 void Program::apply(const std::list<GTLCore::Buffer*>& inputs, GTLCore::Buffer& output, GTLCore::ProgressReport* _report, const GTLCore::ChannelsFlags& flags ) const
 {
+  TEST_GUARD_READ(d)
   GTL_ASSERT(d->srcPixelDescriptions.size() == inputs.size());
   OCTL_ASSERT( (output.size() % (d->dstPixelDescription.bitsSize() / 8)) == 0 );
   OCTL_ASSERT( d->func );
@@ -506,6 +519,7 @@ void Program::apply(const std::list<GTLCore::Buffer*>& inputs, GTLCore::Buffer&
 
 void Program::apply(const GTLCore::AbstractImage& input, GTLCore::AbstractImage& output, GTLCore::ProgressReport* report, const GTLCore::ChannelsFlags& flags ) const
 {
+  TEST_GUARD_READ(d)
   GTLCore::AbstractImage::ConstIterator* itSrc = input.createIterator();
   GTLCore::AbstractImage::Iterator* itDst = output.createIterator();
   
@@ -520,54 +534,70 @@ void Program::apply(const GTLCore::AbstractImage& input, GTLCore::AbstractImage&
 
 void Program::setVarying( const GTLCore::String& _name, const GTLCore::Value& _value )
 {
-  std::map< GTLCore::String, void*>::iterator it = d->varyingsPtr.find( _name );
-  std::map< GTLCore::String, llvm::GlobalVariable*>::iterator it2 = d->varyings.find( _name );
-  if( it != d->varyingsPtr.end() )
-  {
-    void* ptr = it->second;
-    if( it2->second->getType()->getElementType() == llvm::Type::getInt32Ty(d->module->getContext()) )
-    {
-        *(int*)ptr =_value.asInt32();
-    } else if( it2->second->getType()->getElementType() == llvm::Type::getInt1Ty(d->module->getContext()) )
-    {
-        *(bool*)ptr = _value.asBoolean();
-    } else if( it2->second->getType()->getElementType() == llvm::Type::getFloatTy(d->module->getContext()) )
-    {
-        GTL_DEBUG("Set " << _value.asFloat32() << " on ptr " << ptr << " from value = " << *(float*)ptr);
-        *(float*)ptr = _value.asFloat32();
-        GTL_DEBUG( *(float*)ptr );
-    }
+  std::size_t id = varyingId(_name);
+  GTL_ASSERT(id != std::size_t(-1));
+  if( id != std::size_t(-1)) {
+    setVarying(id, _value);
   } else {
     OCTL_DEBUG(" No varying named: " << _name);
   }
 }
 
+void Program::setVarying( std::size_t _id, const GTLCore::Value& _value )
+{
+  TEST_GUARD_WRITE(d)
+  GTL_ASSERT( _id < d->varyings.size());
+  if(_id >= d->varyings.size()) return;
+  void* it = d->varyingsPtr[ _id ];
+  llvm::GlobalVariable* it2 = d->varyings[ _id ];
+  GTLCore::copyValueToPtr(_value, it2->getType()->getElementType(), d->moduleData->llvmLinkedModule()->getContext(), it);
+}
+
 GTLCore::Value Program::varying( const GTLCore::String& _name ) const
 {
-  std::map< GTLCore::String, void*>::iterator it = d->varyingsPtr.find( _name );
-  std::map< GTLCore::String, llvm::GlobalVariable*>::iterator it2 = d->varyings.find( _name );
-  if( it != d->varyingsPtr.end() )
-  {
-    void* ptr = it->second;
-    if( it2->second->getType()->getElementType() == llvm::Type::getInt32Ty(d->module->getContext()) )
-    {
-        return GTLCore::Value( *(gtl_int32*)ptr);
-    }
-    if( it2->second->getType()->getElementType() == llvm::Type::getInt1Ty(d->module->getContext()) )
-    {
-        return GTLCore::Value( *(bool*)ptr);
-    }
-    if( it2->second->getType()->getElementType() == llvm::Type::getFloatTy(d->module->getContext()) )
-    {
-        return GTLCore::Value( *(float*)ptr);
-    }
-    OCTL_DEBUG("Invalid type");
+  std::size_t id = varyingId(_name);
+  GTL_ASSERT(id != std::size_t(-1));
+  if( id == std::size_t(-1)) {
+    OCTL_DEBUG(" No varying named: " << _name);
     return GTLCore::Value();
+  } else {
+    return varying(id);
   }
-  OCTL_DEBUG(" No varying named: '" << _name << "'");
+}
+
+GTLCore::Value Program::varying( std::size_t _id ) const
+{
+  TEST_GUARD_READ(d)
+  GTL_ASSERT( _id < d->varyings.size());
+  if(_id >= d->varyings.size()) return GTLCore::Value();
+  void* ptr = d->varyingsPtr[_id];
+  llvm::GlobalVariable* it2 = d->varyings[_id];
+  if( it2->getType()->getElementType() == llvm::Type::getInt32Ty(d->moduleData->llvmLinkedModule()->getContext()) )
+  {
+      return GTLCore::Value( *(gtl_int32*)ptr);
+  }
+  if( it2->getType()->getElementType() == llvm::Type::getInt1Ty(d->moduleData->llvmLinkedModule()->getContext()) )
+  {
+      return GTLCore::Value( *(bool*)ptr);
+  }
+  if( it2->getType()->getElementType() == llvm::Type::getFloatTy(d->moduleData->llvmLinkedModule()->getContext()) )
+  {
+      return GTLCore::Value( *(float*)ptr);
+  }
+  OCTL_DEBUG("Invalid type");
   return GTLCore::Value();
 }
 
+std::size_t Program::varyingId( const GTLCore::String& _name ) const
+{
+  if(d->varyingsId.find(_name) != d->varyingsId.end())
+  {
+    return d->varyingsId[_name];
+  }
+  return -1;
+}
+
+
 #if 0
 
 void Program::setVarying( const GTLCore::String& _name, const GTLCore::Value& _value )
