@@ -26,7 +26,11 @@
 
 #include "Debug.h"
 #include "TemplateLexer_p.h"
+#ifdef LLVM_27_OR_28
 #include <llvm/System/Path.h>
+#else
+#include <llvm/Support/Path.h>
+#endif
 #include <GTLCore/CompilationMessages.h>
 #include <GTLCore/CompilationMessages_p.h>
 
