@@ -48,6 +48,7 @@
 #include "../Color.h"
 #include "../TypesManager.h"
 
+using namespace GTLCore;
 using namespace GTLCore::AST;
 
 //------------------------------------------//
@@ -61,7 +62,7 @@ llvm::BasicBlock* Expression::generateStatement( LLVMBackend::GenerationContext&
   return egc.currentBasicBlock();
 }
 
-Expression* Expression::fromValue( const GTLCore::Value& _val)
+Expression* Expression::fromValue( const GTLCore::Value& _val, const GTLCore::Type* _declarationType)
 {
   switch( _val.type()->dataType() )
   {
@@ -86,17 +87,28 @@ Expression* Expression::fromValue( const GTLCore::Value& _val)
     {
       if (_val.type() == Type::Color)
       {
+        GTL_ASSERT(_declarationType);
+        GTL_ASSERT(_declarationType->structName() == "color");
         std::vector<Expression*> expressions;
         GTLCore::Color c = _val.asColor();
         expressions.push_back(new AST::NumberExpression<float>(c.red()));
         expressions.push_back(new AST::NumberExpression<float>(c.green()));
         expressions.push_back(new AST::NumberExpression<float>(c.blue()));
         expressions.push_back(new AST::NumberExpression<float>(c.alpha()));
-        return new AST::ConstantCompoundExpression( GTLCore::TypesManager::getVector(Type::Float32, 4), expressions );
+        return new AST::ConstantCompoundExpression( _declarationType, expressions );
+      } else {
+        std::vector<Expression*> expressions;
+        foreach(const Value& val, *_val.asArray())
+        {
+          expressions.push_back(fromValue(val));
+        }
+        
+        return new AST::ConstantCompoundExpression( _val.type(), expressions );
+        
       }
     }
     default:
-      GTL_ABORT("Unimplmeneted");
+      GTL_ABORT("Unimplmeneted: type is " << _val.type()->dataType() << _val);
   }
 }
 
@@ -104,6 +116,11 @@ Expression* Expression::fromValue( const GTLCore::Value& _val)
 //------------- ProxyExpression ------------//
 //------------------------------------------//
 
+ProxyExpression::ProxyExpression( Expression* expr ) : m_clone( expr)
+{
+  GTL_ASSERT(expr);
+}
+
 LLVMBackend::ExpressionResult ProxyExpression::generateValue( LLVMBackend::GenerationContext& _gc, LLVMBackend::ExpressionGenerationContext& _egc) const
 {
   return m_clone->generateValue( _gc, _egc);
@@ -118,7 +135,7 @@ ExpressionResultSP ProxyExpression::generateValue( GenerationVisitor* _generatio
 //------------- GlobalDataExpression ------------//
 //------------------------------------------//
 
-GlobalDataExpression::GlobalDataExpression( Expression* expression ) : m_expression( expression )
+GlobalDataExpression::GlobalDataExpression( Expression* expression ) : m_expression( expression ), m_isreturn(false)
 { }
 
 GlobalDataExpression::~GlobalDataExpression()
@@ -129,11 +146,12 @@ GlobalDataExpression::~GlobalDataExpression()
 LLVMBackend::ExpressionResult GlobalDataExpression::generateValue( LLVMBackend::GenerationContext& _gc, LLVMBackend::ExpressionGenerationContext& _egc) const
 {
   LLVMBackend::ExpressionResult value = m_expression->generateValue( _gc, _egc );
-  if( type()->dataType() == Type::VECTOR )
+  if( m_isreturn) return value;
+  if( value.isConstant() )
   {
-    return value;
-  } else {
     return LLVMBackend::ExpressionResult( new llvm::GlobalVariable( *_gc.llvmModule(), value.constant()->getType(), true, llvm::GlobalValue::InternalLinkage, value.constant(), "" ), type() );
+  } else {
+    return value;
   }
 }
 
