@@ -20,6 +20,7 @@
 #include "Statement.h"
 
 // LLVM
+#include <llvm/ADT/ArrayRef.h>
 #include <llvm/BasicBlock.h>
 #include <llvm/Constants.h>
 #include <llvm/Function.h>
@@ -283,7 +284,7 @@ PrintStatement::~PrintStatement()
 
 llvm::BasicBlock* PrintStatement::generateStatement( LLVMBackend::GenerationContext& _context, llvm::BasicBlock* _bb1 ) const
 {
-  std::vector<const llvm::Type*> params;
+  std::vector<llvm::Type*> params;
   params.push_back( llvm::Type::getInt32Ty(_context.llvmContext()));
   llvm::FunctionType* definitionType = llvm::FunctionType::get( llvm::Type::getVoidTy(_context.llvmContext()), params, true );
   llvm::Function* func = dynamic_cast<llvm::Function*>( _context.llvmModule()->getOrInsertFunction("print", definitionType));
@@ -317,6 +318,6 @@ llvm::BasicBlock* PrintStatement::generateStatement( LLVMBackend::GenerationCont
       GTL_DEBUG("Unknown type for print " << *type);
     }
   }
-  llvm::CallInst::Create(func, values.begin(), values.end(), "", egc.currentBasicBlock());
+  llvm::CallInst::Create(func, values, "", egc.currentBasicBlock());
   return egc.currentBasicBlock();
 }
