@@ -34,6 +34,7 @@
 #include <cstdarg>
 #include "CompilationMessages.h"
 #include "CompilationMessages_p.h"
+#include "OperatorOverloadCenter_p.h"
 
 using namespace GTLCore;
 
@@ -43,6 +44,7 @@ struct CompilerBase::Private {
   std::map< GTLCore::ScopedName, std::list< GTLCore::Function* > > functions;
   std::list<GTLCore::Function*> functionsToDelete;
   ConvertCenter* convertCenter;
+  OperatorOverloadCenter* operatorOverloadCenter;
   ModuleData* moduleData;
   llvm::Module* llvmModule;
 };
@@ -50,10 +52,12 @@ struct CompilerBase::Private {
 CompilerBase::CompilerBase() : d(new Private)
 {
   d->convertCenter = new ConvertCenter;
+  d->operatorOverloadCenter = new OperatorOverloadCenter;
 }
 CompilerBase::~CompilerBase()
 {
   delete d->convertCenter;
+  delete d->operatorOverloadCenter;
   delete d;
 }
 
@@ -159,11 +163,21 @@ bool CompilerBase::declareFunction( const GTLCore::ScopedName& name, GTLCore::Fu
     return true;
 }
 
+bool CompilerBase::declareConstant( const GTLCore::ScopedName& _name, const GTLCore::Type* _type, const GTLCore::Value& _value)
+{
+  return d->moduleData->appendConstant(_name, _type, _value);
+}
+
 ConvertCenter* CompilerBase::convertCenter()
 {
   return d->convertCenter;
 }
 
+OperatorOverloadCenter* CompilerBase::operatorOverloadCenter()
+{
+  return d->operatorOverloadCenter;
+}
+
 std::list<GTLCore::Function*>& CompilerBase::functionsToDelete()
 {
   return d->functionsToDelete;
