@@ -25,7 +25,11 @@
 #include <llvm/Module.h>
 #include <llvm/Value.h>
 #include <llvm/Type.h>
+#ifdef LLVM_27_OR_28
 #include <llvm/System/Path.h>
+#else
+#include <llvm/Support/Path.h>
+#endif
 #include <llvm/Support/raw_ostream.h>
 
 #include "Function.h"
@@ -331,7 +335,7 @@ namespace GTLCore {
         ostr <<"VOID";
         break;
       case Type::STRUCTURE:
-        ostr <<"STRUCTURE";
+        ostr <<"STRUCTURE("  << type.structName() << ")";
         break;
       case Type::ARRAY:
         ostr <<"ARRAY[" << *type.embeddedType() << "]";
@@ -451,7 +455,8 @@ namespace llvm {
   {                                                                       \
     std::string str;                                                      \
     llvm::raw_string_ostream rs(str);                                     \
-    rs << val;                                                            \
+    rs << &val;                                                            \
+    rs.flush();                                                           \
     ostr << str;                                                          \
     return ostr;                                                          \
   }
