@@ -124,7 +124,7 @@ llvm::Constant* CodeGenerator::floatToConstant(llvm::LLVMContext& _llvmContext,
 
 llvm::Constant* CodeGenerator::nullPointerToConstant(llvm::LLVMContext& _llvmContext)
 {
-  return llvm::ConstantPointerNull::get( static_cast<const llvm::PointerType*>( Type::Pointer->d->type(_llvmContext) ) );
+  return llvm::ConstantPointerNull::get( llvm::cast<llvm::PointerType>( Type::Pointer->d->type(_llvmContext) ) );
 }
 
 llvm::Constant* CodeGenerator::valueToConstant( LLVMBackend::GenerationContext& _gc, const Value& v)
@@ -184,7 +184,7 @@ llvm::Constant* CodeGenerator::constantsToArray( LLVMBackend::GenerationContext&
   
   arrayStruct.push_back( iptr );
   
-  const llvm::StructType* structType = dynamic_cast<const llvm::StructType*>( _type->d->type(_gc.llvmContext()) );
+  llvm::StructType* structType = llvm::cast<llvm::StructType>( _type->d->type(_gc.llvmContext()) );
   GTL_ASSERT( structType );
   return llvm::ConstantStruct::get( structType, arrayStruct );
 }
@@ -192,16 +192,16 @@ llvm::Constant* CodeGenerator::constantsToArray( LLVMBackend::GenerationContext&
 llvm::Constant* CodeGenerator::constantsToStructure( LLVMBackend::GenerationContext& _gc, const std::vector< llvm::Constant* >& _constants, const Type* _type )
 {
   GTL_ASSERT(_constants.size() == _type->countStructDataMembers() + STRUCT_FIRST_ELEMENT);
-  const llvm::StructType* structType = dynamic_cast<const llvm::StructType*>( _type->d->type(_gc.llvmContext()) );
+  llvm::StructType* structType = llvm::cast<llvm::StructType>( _type->d->type(_gc.llvmContext()) );
   return llvm::ConstantStruct::get( structType, _constants );
 }
 
 llvm::Constant* CodeGenerator::constantsToVector( LLVMBackend::GenerationContext& _gc, const std::vector< llvm::Constant* >& _constants, const Type* _type )
 {
-  const llvm::VectorType* vectorType = dynamic_cast<const llvm::VectorType*>( _type->d->type(_gc.llvmContext()) );
+  llvm::VectorType* vectorType = llvm::cast<llvm::VectorType>( _type->d->type(_gc.llvmContext()) );
   GTL_ASSERT( vectorType );
   GTL_ASSERT( not _constants.empty());
-  return llvm::ConstantVector::get( vectorType, _constants );
+  return llvm::ConstantVector::get( _constants );
 }
 
 Value CodeGenerator::constantToValue(llvm::Constant* constant, const Type* _type)
@@ -289,10 +289,10 @@ llvm::Value* CodeGenerator::createVector( const Type* _vecType, llvm::Constant*
     inits.push_back( _val );
   }
   GTL_ASSERT( not inits.empty());
-  return llvm::ConstantVector::get( static_cast<const llvm::VectorType*>( _vecType->d->type( _val->getContext() )), inits );
+  return llvm::ConstantVector::get( inits );
 }
 
-llvm::Value* CodeGenerator::createVector( int _size, llvm::Constant* _val, const Type* _valType)
+llvm::Value* CodeGenerator::createVector( int _size, llvm::Constant* _val)
 {
   std::vector<llvm::Constant*> inits;
   for( int i = 0; i < _size; ++i)
@@ -300,7 +300,7 @@ llvm::Value* CodeGenerator::createVector( int _size, llvm::Constant* _val, const
     inits.push_back( _val );
   }
   GTL_ASSERT( not inits.empty());
-  return llvm::ConstantVector::get( static_cast<const llvm::VectorType*>( TypesManager::getVector( _valType, _size)->d->type( _val->getContext() )), inits );
+  return llvm::ConstantVector::get( inits );
 }
 
 llvm::Value* CodeGenerator::convertPointerToCharP(llvm::BasicBlock* _currentBlock, llvm::Value* _value)
@@ -308,7 +308,7 @@ llvm::Value* CodeGenerator::convertPointerToCharP(llvm::BasicBlock* _currentBloc
   return convertPointerTo( _currentBlock, _value, llvm::Type::getInt8Ty(_value->getContext()) );
 }
 
-llvm::Value* CodeGenerator::convertPointerTo(llvm::BasicBlock* _currentBlock, llvm::Value* _value, const llvm::Type* _type)
+llvm::Value* CodeGenerator::convertPointerTo(llvm::BasicBlock* _currentBlock, llvm::Value* _value, llvm::Type* _type)
 {
   return new llvm::BitCastInst( _value, llvm::PointerType::get( _type, 0 ), "", _currentBlock );
 }
@@ -354,7 +354,7 @@ llvm::Value* CodeGenerator::convertValueTo(llvm::BasicBlock* _currentBlock, llvm
 
 llvm::Constant* CodeGenerator::convertConstantTo(llvm::Constant* _constant, const Type* _constantType, const Type* _targetType)
 {
-  const llvm::Type* dstType = _targetType->d->type(_constant->getContext());
+  llvm::Type* dstType = _targetType->d->type(_constant->getContext());
   GTL_DEBUG( *_constant->getType() << " " << *dstType << " "<< *_targetType );
   if(_constant->getType() == dstType )
   { // Don't cast the value as it has already the correct type
@@ -371,7 +371,7 @@ llvm::Constant* CodeGenerator::convertConstantTo(llvm::Constant* _constant, cons
     }
     // store back
     GTL_ASSERT( not constants.empty());
-    return llvm::ConstantVector::get( llvm::cast<const llvm::VectorType>(dstType), constants );
+    return llvm::ConstantVector::get( constants );
   }
   GTL_ASSERT( _targetType->d->type(_constant->getContext())->isFirstClassType() );
   GTL_ASSERT( _constant->getType()->isFirstClassType() );
@@ -384,29 +384,30 @@ llvm::Constant* CodeGenerator::convertConstantTo(llvm::Constant* _constant, cons
 
 llvm::Value* CodeGenerator::convertToHalf( LLVMBackend::GenerationContext& _generationContext, llvm::BasicBlock* currentBlock, llvm::Value* value, const Type* _valueType )
 {
-  std::vector<const llvm::Type*> llvmArguments;
+  std::vector< llvm::Type*> llvmArguments;
   llvmArguments.push_back( llvm::Type::getFloatTy(_generationContext.llvmContext()) );
   llvm::Function* func = dynamic_cast<llvm::Function*>( _generationContext.llvmModule()->getOrInsertFunction( "_Z10float2halff",
       llvm::FunctionType::get( llvm::Type::getInt16Ty(_generationContext.llvmContext()) , llvmArguments, false ) ) );
   std::vector<llvm::Value*> convertedParams;
   convertedParams.push_back( convertValueTo( currentBlock, value, _valueType, Type::Float32 ) );
-  return llvm::CallInst::Create( func, convertedParams.begin(), convertedParams.end(), "", currentBlock );
+  return llvm::CallInst::Create( func, convertedParams, "", currentBlock );
 }
 
 llvm::Value* CodeGenerator::convertFromHalf( LLVMBackend::GenerationContext& _generationContext, llvm::BasicBlock* currentBlock, llvm::Value* value)
 {
-  std::vector<const llvm::Type*> llvmArguments;
+  std::vector<llvm::Type*> llvmArguments;
   llvmArguments.push_back( llvm::Type::getInt16Ty(_generationContext.llvmContext()) );
   llvm::Function* func = dynamic_cast<llvm::Function*>( _generationContext.llvmModule()->getOrInsertFunction( "_Z10half2floatt",
       llvm::FunctionType::get( llvm::Type::getFloatTy(_generationContext.llvmContext()) , llvmArguments, false ) ) );
   std::vector<llvm::Value*> convertedParams;
   convertedParams.push_back( convertValueTo( currentBlock, value, Type::Float16, Type::Integer16 ) );
-  return llvm::CallInst::Create( func, convertedParams.begin(), convertedParams.end(), "", currentBlock );
+  return llvm::CallInst::Create( func, convertedParams, "", currentBlock );
   
 }
 
 llvm::Value* CodeGenerator::convertArrayToVector(LLVMBackend::GenerationContext& gc, LLVMBackend::ExpressionGenerationContext& _egc, llvm::Value* value, const Type* valueType, const Type* type)
 {
+  
   // int arraySize = value->size;
   // if( arraySize > type->vectorSize() ) arraySize = type->vectorSize();
   // 'type' result;
@@ -418,7 +419,7 @@ llvm::Value* CodeGenerator::convertArrayToVector(LLVMBackend::GenerationContext&
   // int arraySize = value->size;
   llvm::Value* arraySize = accessArraySize(_egc.currentBasicBlock(), value);
   llvm::Value* vectorSize = integerToConstant( gc.llvmContext(), gtl_uint32(type->vectorSize()) );
-  llvm::Value* arraySizeAlloc = new llvm::AllocaInst( llvm::Type::getInt32Ty(gc.llvmContext()), "", _egc.currentBasicBlock() );
+  llvm::Value* arraySizeAlloc = new llvm::AllocaInst( llvm::Type::getInt32Ty(gc.llvmContext()), "arraySizeAlloc", _egc.currentBasicBlock() );
   
   // arraySize > type->vectorSize()
   llvm::Value* aStVComp = createStrictSupperiorExpression(_egc.currentBasicBlock(),
@@ -433,8 +434,8 @@ llvm::Value* CodeGenerator::convertArrayToVector(LLVMBackend::GenerationContext&
   llvm::BasicBlock* after = gc.createBasicBlock();
   createIfElseStatement( _egc.currentBasicBlock(), aStVComp, Type::Boolean, block1, block1, block2, block2, after);
 
-  arraySize = new llvm::LoadInst( arraySizeAlloc, "", after );
-  llvm::Value* result = createVector( type, floatToConstant( gc.llvmContext(), 0.0 ) );
+  arraySize = new llvm::LoadInst( arraySizeAlloc, "loadArraySizeAlloc", after );
+  llvm::Value* result = new llvm::AllocaInst( type->d->type(gc.llvmContext()), "vector", after );
   
   // int i = 0;
   VariableNG idx(Type::Integer32, false, false );
@@ -445,15 +446,15 @@ llvm::Value* CodeGenerator::convertArrayToVector(LLVMBackend::GenerationContext&
   //   result[i] = array[i];
   llvm::BasicBlock* forBlock = gc.createBasicBlock();
   llvm::Value* cidx = idx.get( gc, forBlock );
-  llvm::Value* val = accessArrayValueNoClamp( forBlock, value, cidx );
-  llvm::InsertElementInst::Create(result, val, cidx, "", forBlock );
+  llvm::Value* val = new llvm::LoadInst( accessArrayValueNoClamp( forBlock, value, cidx ), "", forBlock);
+  new llvm::StoreInst( llvm::InsertElementInst::Create(new llvm::LoadInst(result, "", forBlock), val, cidx, "", forBlock ), result, "", forBlock );
   
   // for(int i = 0; i < arraySize; ++i )
   llvm::BasicBlock* afterFor = createIterationForStatement(gc, after, &idx, arraySize, Type::Integer32, forBlock, forBlock);
   
   _egc.setCurrentBasicBlock(afterFor);
   
-  return result;
+  return new llvm::LoadInst(result, "", afterFor);
 }
 
 llvm::Constant* CodeGenerator::convertConstantArrayToVector(llvm::Constant* constant, const Type* constantType, const Type* type)
@@ -465,7 +466,7 @@ llvm::Constant* CodeGenerator::convertConstantArrayToVector(llvm::Constant* cons
   GTL_ASSERT(constantType->dataType() == Type::ARRAY );
   GTL_ASSERT(type->dataType() == Type::VECTOR );
   GTL_ASSERT(arrayConstant->getNumOperands() == 3 );
-  llvm::Constant* values = static_cast<llvm::Constant*>(arrayConstant->getOperand(2)->getOperand(0)->getOperand(0));
+  llvm::Constant* values = llvm::cast<llvm::Constant>(llvm::cast<llvm::Constant>(arrayConstant->getOperand(2)->getOperand(0))->getOperand(0));
   GTL_DEBUG(*values);
   GTL_ASSERT(dynamic_cast<llvm::ConstantArray*>(values) );
   std::vector<llvm::Constant*> constants;
@@ -480,7 +481,7 @@ llvm::Constant* CodeGenerator::convertConstantArrayToVector(llvm::Constant* cons
   }
   GTL_ASSERT( constants.size() == type->vectorSize() );
   GTL_ASSERT( not constants.empty());
-  return llvm::ConstantVector::get( static_cast<const llvm::VectorType*>( type->d->type(constant->getContext()) ), constants);
+  return llvm::ConstantVector::get( constants);
 }
 
 llvm::Value* CodeGenerator::vectorValueAt( llvm::BasicBlock* _currentBlock, llvm::Value* _vector, llvm::Value* _index)
@@ -665,6 +666,8 @@ llvm::Value* CodeGenerator::createDivisionExpression(llvm::BasicBlock* currentBl
            and lhsType->embeddedType()->dataType() == Type::FLOAT32 ) )
   {
     return llvm::BinaryOperator::CreateFDiv(v1, v2, "", currentBlock );
+  } else if( lhsType->isSigned() and rhsType->isSigned()) {
+    return llvm::BinaryOperator::CreateUDiv(v1, v2, "", currentBlock );
   } else {
     return llvm::BinaryOperator::CreateSDiv(v1, v2, "", currentBlock );
   }
@@ -676,6 +679,8 @@ llvm::Constant* CodeGenerator::createDivisionExpression( llvm::Constant* lhs, co
   if( v1->getType()->isFloatingPointTy() )
   {
     return llvm::ConstantExpr::getFDiv( v1, v2);
+  } else if( lhsType->isSigned() and rhsType->isSigned()) {
+    return llvm::ConstantExpr::getUDiv(v1, v2 );
   } else {
     return llvm::ConstantExpr::getSDiv( v1, v2);
   }
@@ -692,13 +697,21 @@ ExpressionResult CodeGenerator::createDivisionExpression( llvm::BasicBlock* curr
 llvm::Value* CodeGenerator::createModuloExpression(llvm::BasicBlock* currentBlock, llvm::Value* lhs, const Type* lhsType, llvm::Value* rhs, const Type* rhsType)
 {
   UNIFORMIZE_VALUE_TYPES( lhs, rhs );
-  return llvm::BinaryOperator::CreateSRem( v1, v2, "", currentBlock );
+  if( lhsType->isSigned() or rhsType->isSigned()) {
+    return llvm::BinaryOperator::CreateSRem( v1, v2, "", currentBlock );
+  } else {
+    return llvm::BinaryOperator::CreateURem( v1, v2, "", currentBlock );
+  }
 }
 
 llvm::Constant* CodeGenerator::createModuloExpression( llvm::Constant* lhs, const Type* lhsType, llvm::Constant* rhs, const Type* rhsType)
 {
   UNIFORMIZE_CONSTANT_TYPES( lhs, rhs );
-  return llvm::ConstantExpr::getSRem( v1, v2);
+  if( lhsType->isSigned() or rhsType->isSigned()) {
+    return llvm::ConstantExpr::getSRem( v1, v2);
+  } else {
+    return llvm::ConstantExpr::getURem( v1, v2);
+  }
 }
 
 ExpressionResult CodeGenerator::createModuloExpression( llvm::BasicBlock* currentBlock, ExpressionResult lhs, const Type* lhsType, ExpressionResult rhs, const Type* rhsType)
@@ -831,7 +844,7 @@ ExpressionResult CodeGenerator::createMinusExpression( llvm::BasicBlock* current
 
 llvm::Value* CodeGenerator::createMinusExpression(llvm::BasicBlock* currentBlock, llvm::Value* rhs, const Type* rhsType)
 {
-  if(rhsType->isFloatingPoint() or (rhsType->dataType() == Type::VECTOR and rhsType->embeddedType()->isFloatingPoint() ))
+  if(rhsType->isFloatingPoint())
   {
     return llvm::BinaryOperator::CreateFNeg( rhs, "", currentBlock );
   } else {
@@ -841,7 +854,7 @@ llvm::Value* CodeGenerator::createMinusExpression(llvm::BasicBlock* currentBlock
 
 llvm::Constant* CodeGenerator::createMinusExpression( llvm::Constant* rhs, const Type* rhsType)
 {
-  if(rhsType->isFloatingPoint() or (rhsType->dataType() == Type::VECTOR and rhsType->embeddedType()->isFloatingPoint() ))
+  if(rhsType->isFloatingPoint())
   {
     return llvm::ConstantExpr::getFNeg( rhs);
   } else {
@@ -881,15 +894,27 @@ llvm::Constant* CodeGenerator::createTildeExpression( llvm::Constant* rhs, const
 
 llvm::Value* CodeGenerator::accessArrayValueClamp( LLVMBackend::GenerationContext& generationContext, LLVMBackend::ExpressionGenerationContext& _expressionGenerationContext, llvm::Value* _pointer, llvm::Value* _index )
 {
-  llvm::BasicBlock* currentBlock = _expressionGenerationContext.currentBasicBlock();
   // Clamp
-  llvm::Value* indexValuePtr = new llvm::AllocaInst(llvm::Type::getInt32Ty(generationContext.llvmContext()), "ClampedPlacehold", currentBlock);
-  new llvm::StoreInst(_index, indexValuePtr, "Store for clamping", currentBlock);
-  currentBlock = createClampExpression(generationContext, currentBlock, indexValuePtr, Type::Integer32, integerToConstant(generationContext.llvmContext(), gtl_uint32(0)), 
-                                       createSubstractionExpression(currentBlock, accessArraySize(currentBlock, _pointer), Type::Integer32, integerToConstant(generationContext.llvmContext(), gtl_uint32(1)), Type::Integer32) );
+  
+  llvm::Value* min = integerToConstant(generationContext.llvmContext(), gtl_uint32(0));
+  llvm::Value* max = createSubstractionExpression(_expressionGenerationContext.currentBasicBlock(),
+                                                  accessArraySize(_expressionGenerationContext.currentBasicBlock(), _pointer),
+                                                  Type::Integer32,
+                                                  integerToConstant(generationContext.llvmContext(), gtl_uint32(1)),
+                                                  Type::Integer32 );
+  
+  _index = clampValue(generationContext, _expressionGenerationContext, _index, min, max );
+  return accessArrayValueNoClamp(_expressionGenerationContext.currentBasicBlock(), _pointer, _index);
+}
+
+llvm::Value* CodeGenerator::clampValue( LLVMBackend::GenerationContext& _generationContext, LLVMBackend::ExpressionGenerationContext& _expressionGenerationContext, llvm::Value* _value, llvm::Value* _min, llvm::Value* _max)
+{
+  llvm::BasicBlock* currentBlock = _expressionGenerationContext.currentBasicBlock();
+  llvm::Value* indexValuePtr = new llvm::AllocaInst(llvm::Type::getInt32Ty(_generationContext.llvmContext()), "ClampedPlacehold", currentBlock);
+  new llvm::StoreInst(_value, indexValuePtr, "Store for clamping", currentBlock);
+  currentBlock = createClampExpression(_generationContext, currentBlock, indexValuePtr, Type::Integer32, _min, _max);
   _expressionGenerationContext.setCurrentBasicBlock(currentBlock);
-  _index = new llvm::LoadInst(indexValuePtr, "Loadnewindex", currentBlock);
-  return accessArrayValueNoClamp(currentBlock, _pointer, _index);
+  return new llvm::LoadInst(indexValuePtr, "Loadnewindex", currentBlock);
 }
 
 llvm::Value* CodeGenerator::accessArrayValueNoClamp( llvm::BasicBlock* _currentBlock, llvm::Value* _pointer, llvm::Value* _index )
@@ -900,7 +925,7 @@ llvm::Value* CodeGenerator::accessArrayValueNoClamp( llvm::BasicBlock* _currentB
   indexes.push_back( llvm::ConstantInt::get(llvm::Type::getInt32Ty(_pointer->getContext()), ArrayWrap::POS_DATA)); // Access the data of the array
   
   // Select the pointer
-  llvm::Value* iptr = llvm::GetElementPtrInst::Create( _pointer, indexes.begin(), indexes.end(), "", _currentBlock);
+  llvm::Value* iptr = llvm::GetElementPtrInst::Create( _pointer, indexes, "", _currentBlock);
   // pointer on the values
   llvm::Value* iptr_val = new llvm::LoadInst( iptr, "load array data pointer", _currentBlock);
   return llvm::GetElementPtrInst::Create( iptr_val, _index, "point to the value in the data pointer", _currentBlock);
@@ -911,12 +936,12 @@ llvm::Value* CodeGenerator::accessArraySizePointer( llvm::BasicBlock* _currentBl
   std::vector<llvm::Value*> indexes;
   indexes.push_back( llvm::ConstantInt::get(llvm::Type::getInt32Ty(_pointer->getContext()), 0)); // Access the structure of the array
   indexes.push_back( llvm::ConstantInt::get(llvm::Type::getInt32Ty(_pointer->getContext()), ArrayWrap::POS_SIZE)); // Access the size of the array
-  return llvm::GetElementPtrInst::Create( _pointer, indexes.begin(), indexes.end(), "", _currentBlock);
+  return llvm::GetElementPtrInst::Create( _pointer, indexes, "", _currentBlock);
 }
 
 llvm::Value* CodeGenerator::accessArraySize( llvm::BasicBlock* _currentBlock, llvm::Value* _pointer )
 {
-  return new llvm::LoadInst( accessArraySizePointer(_currentBlock, _pointer), "", _currentBlock);
+  return new llvm::LoadInst( accessArraySizePointer(_currentBlock, _pointer), "accessArraySize", _currentBlock);
 }
 
 ExpressionResult CodeGenerator::callFunction( LLVMBackend::GenerationContext& _gc, LLVMBackend::ExpressionGenerationContext& egc, const Function* _function, const std::list<AST::Expression*>& _arguments )
@@ -947,7 +972,7 @@ ExpressionResult CodeGenerator::callFunction( LLVMBackend::GenerationContext& _g
       AST::AccessorExpression* aexp = dynamic_cast<AST::AccessorExpression*>( *it );
       GTL_ASSERT( aexp );
       llvm::Value* pointer = aexp->pointer( _gc, egc );
-      GTL_ASSERT( pointer->getType() == llvm::PointerType::get( oparam_it->type()->d->type(_gc.llvmContext()), 0) );
+      GTL_CHECK_LLVM_EQUAL( pointer->getType(), llvm::PointerType::get( oparam_it->type()->d->type(_gc.llvmContext()), 0) );
       convertedParams.push_back( pointer );
       GTL_DEBUG(*pointer->getType() << " " << *oparam_it->type()->d->type(_gc.llvmContext()) << " " << pointer );
     } else {
@@ -1006,7 +1031,7 @@ ExpressionResult CodeGenerator::callFunction( LLVMBackend::GenerationContext& _g
   }
 #endif
   
-  llvm::CallInst *CallInst = llvm::CallInst::Create(llvmFunction, convertedParams.begin(), convertedParams.end(), "", egc.currentBasicBlock());
+  llvm::CallInst *CallInst = llvm::CallInst::Create(llvmFunction, convertedParams, "", egc.currentBasicBlock());
   CallInst->setTailCall(false);
   llvm::Value* result = CallInst;
   if(_function->d->isReturnedAsPointer())
@@ -1022,8 +1047,9 @@ llvm::Value* CodeGenerator::countFieldPointer( llvm::BasicBlock* _currentBlock,
   std::vector<llvm::Value*> indexes;
   indexes.push_back( llvm::ConstantInt::get(llvm::Type::getInt32Ty(_pointer->getContext()), 0)); // Access the structure of the array/structure
   indexes.push_back( llvm::ConstantInt::get(llvm::Type::getInt32Ty(_pointer->getContext()), 0)); // Access the count field of the array/structure
+  GTL_DEBUG(*_pointer << *_pointer->getType());
   // Select the pointer
-  return llvm::GetElementPtrInst::Create( _pointer, indexes.begin(), indexes.end(), "", _currentBlock);
+  return llvm::GetElementPtrInst::Create( _pointer, indexes, "", _currentBlock);
 }
 
 void CodeGenerator::setCountFieldOf( llvm::BasicBlock* _currentBlock, llvm::Value* _pointer, llvm::Value* _value )
@@ -1050,7 +1076,7 @@ llvm::Value* CodeGenerator::incrementCountFieldOf( llvm::BasicBlock* _currentBlo
 
 llvm::Function* createGtlMallocFunction( LLVMBackend::GenerationContext& _gc )
 {
-  std::vector<const llvm::Type*> memcpyTyArgs;
+  std::vector< llvm::Type*> memcpyTyArgs;
   memcpyTyArgs.push_back(llvm::Type::getInt32Ty(_gc.llvmContext()));
   llvm::FunctionType* memcpyTy = llvm::FunctionType::get( llvm::PointerType::get(llvm::IntegerType::getInt8Ty(_gc.llvmContext()), 0), memcpyTyArgs, false);
   
@@ -1062,7 +1088,7 @@ llvm::Function* createGtlMallocFunction( LLVMBackend::GenerationContext& _gc )
 
 llvm::Function* createGtlFreeFunction( LLVMBackend::GenerationContext& _gc )
 {
-  std::vector<const llvm::Type*> memcpyTyArgs;
+  std::vector< llvm::Type*> memcpyTyArgs;
   memcpyTyArgs.push_back(llvm::PointerType::get(llvm::IntegerType::getInt8Ty(_gc.llvmContext()), 0));
   llvm::FunctionType* memcpyTy = llvm::FunctionType::get( llvm::Type::getVoidTy(_gc.llvmContext()), memcpyTyArgs, false);
   
@@ -1073,7 +1099,7 @@ llvm::Function* createGtlFreeFunction( LLVMBackend::GenerationContext& _gc )
 }
 #endif
 
-llvm::Value* CodeGenerator::allocateMemory( LLVMBackend::GenerationContext& _gc, const llvm::Type* _type, llvm::Value* _size, llvm::BasicBlock* _bb)
+llvm::Value* CodeGenerator::allocateMemory( LLVMBackend::GenerationContext& _gc, llvm::Type* _type, llvm::Value* _size, llvm::BasicBlock* _bb)
 {
 #ifdef _USE_GTL_MM_
   llvm::Function* func = createGtlMallocFunction( _gc );
@@ -1084,7 +1110,7 @@ llvm::Value* CodeGenerator::allocateMemory( LLVMBackend::GenerationContext& _gc,
     llvm::CastInst::CreateIntegerCast( llvm::ConstantExpr::getSizeOf( _type ),
                                        llvm::Type::getInt32Ty(_gc.llvmContext()), false, "", _bb ), "", _bb  ) );
   GTL_DEBUG( *func );
-  llvm::CallInst *CallFunc = llvm::CallInst::Create(func, params.begin(), params.end(), "", _bb);
+  llvm::CallInst *CallFunc = llvm::CallInst::Create(func, params, "", _bb);
   CallFunc->setTailCall(false);
   return convertPointerTo( _bb, CallFunc, _type );
 #else
@@ -1102,7 +1128,7 @@ void CodeGenerator::freeMemory( LLVMBackend::GenerationContext& _gc, llvm::Value
   std::vector<llvm::Value*> params;
   params.push_back( convertPointerTo(_bb, _ptr, llvm::IntegerType::getInt8Ty(_gc.llvmContext()) ) );
   GTL_DEBUG(*params[0]);
-  llvm::CallInst *CallFunc = llvm::CallInst::Create(func, params.begin(), params.end(), "", _bb);
+  llvm::CallInst *CallFunc = llvm::CallInst::Create(func, params, "", _bb);
   CallFunc->setTailCall(false);
 #else
 #ifndef _USE_OS_MM_
@@ -1114,7 +1140,7 @@ void CodeGenerator::freeMemory( LLVMBackend::GenerationContext& _gc, llvm::Value
 
 llvm::Function* createProgressReportFunction( llvm::Module* _module, const String& name )
 {
-  std::vector<const llvm::Type*> memcpyTyArgs;
+  std::vector< llvm::Type*> memcpyTyArgs;
   memcpyTyArgs.push_back(llvm::PointerType::get(llvm::Type::getInt8Ty(_module->getContext()), 0));
   llvm::FunctionType* memcpyTy = llvm::FunctionType::get( llvm::Type::getVoidTy(_module->getContext()), memcpyTyArgs, false);
   
@@ -1133,13 +1159,34 @@ llvm::BasicBlock* callProgressReport( LLVMBackend::GenerationContext& _gc, llvm:
   
   std::vector<llvm::Value*> params;
   params.push_back( _a );
-  llvm::CallInst *CallFunc = llvm::CallInst::Create(_function, params.begin(), params.end(), "", ifblock);
+  llvm::CallInst *CallFunc = llvm::CallInst::Create(_function, params, "", ifblock);
   CallFunc->setTailCall(false);
+
+  // Test if canceled
+  llvm::BasicBlock* afterIfInterruptedBlock = _gc.createBasicBlock();
+  
+  {
+    std::vector< llvm::Type*> memcpyTyArgs;
+    memcpyTyArgs.push_back(llvm::PointerType::get(llvm::Type::getInt8Ty(_gc.llvmContext()), 0));
+    llvm::FunctionType* memcpyTy = llvm::FunctionType::get( llvm::Type::getInt1Ty(_gc.llvmContext()), memcpyTyArgs, false);
+    
+    llvm::Function* func_interrupted = (llvm::Function*)_gc.llvmModule()->getOrInsertFunction(
+          "gtl_progress_report_interrupted", memcpyTy); // (external, no body)
+    func_interrupted->setCallingConv(llvm::CallingConv::C);
+    
+    llvm::BasicBlock* ifInterruptedBlock = _gc.createBasicBlock();
+    llvm::ReturnInst::Create(_gc.llvmContext(), ifInterruptedBlock);
+    
+    llvm::CallInst *CallInterruptedFunc = llvm::CallInst::Create(func_interrupted, params, "", ifblock);
+    CallInterruptedFunc->setTailCall(false);
+    
+    CodeGenerator::createIfStatement(ifblock, CallInterruptedFunc, Type::Boolean, ifInterruptedBlock, ifInterruptedBlock, afterIfInterruptedBlock);
+    
+  }
   
   llvm::BasicBlock* afterBlock = _gc.createBasicBlock();
   
-  
-  CodeGenerator::createIfStatement(_bb, test, Type::Boolean, ifblock, ifblock, afterBlock);
+  CodeGenerator::createIfStatement(_bb, test, Type::Boolean, ifblock, afterIfInterruptedBlock, afterBlock);
   
   return afterBlock;  
 }
