@@ -108,7 +108,7 @@ namespace LLVMBackend {
        * Create a vector of @p _size values equal to @p _val
        * @param _valType the type of the @p _val
        */
-      static llvm::Value* createVector( int _size, llvm::Constant* _val, const GTLCore::Type* _valType);
+      static llvm::Value* createVector( int _size, llvm::Constant* _val);
       /**
        * Create a function and add it to the module.
        * @param name name of the function
@@ -120,7 +120,7 @@ namespace LLVMBackend {
       /**
        * Convert the pointer contained in @p value to a pointer of type @p type
        */
-      static llvm::Value* convertPointerTo(llvm::BasicBlock* currentBlock, llvm::Value* value, const llvm::Type* type);
+      static llvm::Value* convertPointerTo(llvm::BasicBlock* _currentBlock, llvm::Value* _value, llvm::Type* _type);
       static llvm::Value* convertValueTo(llvm::BasicBlock* currentBlock, llvm::Value* value, const GTLCore::Type* valueType, const GTLCore::Type* type);
       static llvm::Constant* convertConstantTo(llvm::Constant* constant, const GTLCore::Type* constantType, const GTLCore::Type* type);
       static llvm::Value* convertToHalf( LLVMBackend::GenerationContext& generationContext, llvm::BasicBlock* currentBlock, llvm::Value* value, const GTLCore::Type* _valueType);
@@ -266,6 +266,10 @@ namespace LLVMBackend {
        * @param _valuePtr pointer to the value
        */
       static llvm::BasicBlock* createClampExpression( LLVMBackend::GenerationContext& _context, llvm::BasicBlock* _current, llvm::Value* _valuePtr, const GTLCore::Type* _type, llvm::Value* _min, llvm::Value* _max);
+      /**
+       * Clamp a value
+       */
+      static llvm::Value* clampValue( LLVMBackend::GenerationContext& _context, LLVMBackend::ExpressionGenerationContext& _expressionGenerationContext, llvm::Value* _value, llvm::Value* _min, llvm::Value* _max);
     public:
       /**
        * Set the count field of a Structure/Array to a given value.
@@ -295,7 +299,25 @@ namespace LLVMBackend {
     public:
       ExpressionResult callFunction( LLVMBackend::GenerationContext& _gc, LLVMBackend::ExpressionGenerationContext& egc, const GTLCore::Function* _function, const std::list<GTLCore::AST::Expression*>& m_arguments );
     public:
-      static llvm::Value* allocateMemory( LLVMBackend::GenerationContext& _gc, const llvm::Type* _type, llvm::Value* _a, llvm::BasicBlock* _bb);
+      /**
+       * This function allocate memory using the @ref MemoryManager (or malloc depending on compilation options).
+       * For safety reason, only use @ref freeMemory to free memory allocated using @ref allocateMemory.
+       * For thread safety, only allocate/free a given memory block from the same thread, however the memory can
+       * be used from different threads, given that the usual threading precautions are taken.
+       *
+       * @param _type the type of the object to allocate
+       * @param _a the number of elements to allocate
+       * @return a pointer to the allocated memory
+       */
+      static llvm::Value* allocateMemory( LLVMBackend::GenerationContext& _gc, llvm::Type* _type, llvm::Value* _size, llvm::BasicBlock* _bb);
+      /**
+       * This function free memory using the @ref MemoryManager (or malloc depending on compilation options).
+       * For safety reason, only use @ref freeMemory to free memory allocated using @ref allocateMemory.
+       * For thread safety, only allocate/free a given memory block from the same thread, however the memory can
+       * be used from different threads, given that the usual threading precautions are taken.
+       *
+       * @param _ptr to the memory
+       */
       static void freeMemory( LLVMBackend::GenerationContext& _gc, llvm::Value* _ptr, llvm::BasicBlock* _bb);
     public:
       static llvm::BasicBlock* callProgressReportNextRow( LLVMBackend::GenerationContext& _gc, llvm::Value* _a, llvm::BasicBlock* _bb);
