@@ -28,7 +28,11 @@
 #include <llvm/Analysis/Verifier.h>
 #include <llvm/Transforms/Utils/Cloning.h>
 #include <llvm/Linker.h>
+#ifdef LLVM_27_OR_28
 #include <llvm/System/Path.h>
+#else
+#include <llvm/Support/Path.h>
+#endif
 
 #include "Debug.h"
 #include "Parameter.h"
@@ -93,9 +97,9 @@ const std::list<Function*>* ModuleData::function(const String& _currentNameSpace
   return 0;
 }
 
-const std::list<Function*>* ModuleData::function(const ScopedName& name)
+const std::list<Function*>* ModuleData::function(const ScopedName& name) const
 {
-  std::map<ScopedName, std::list<Function*>* >::iterator it = m_functions.find(name);
+  std::map<ScopedName, std::list<Function*>* >::const_iterator it = m_functions.find(name);
   if( it == m_functions.end())
   {
     return 0;
@@ -193,6 +197,28 @@ const GTLCore::Function* ModuleData::function(const GTLCore::String& _nameSpace,
   return bestFunction;
 }
 
+bool ModuleData::appendConstant(const GTLCore::ScopedName& name, const GTLCore::Type* type, const GTLCore::Value& val)
+{
+  if (m_constants.find(name) == m_constants.end())
+  {
+    m_constants[name] = type;
+    m_constantsValue[name] = val;
+    return true;
+  } else {
+    return false;
+  }
+}
+
+const std::map<GTLCore::ScopedName, const GTLCore::Type*>& ModuleData::constants() const
+{
+  return m_constants;
+}
+
+const std::map<ScopedName, GTLCore::Value>& ModuleData::constantsValue() const
+{
+  return m_constantsValue;
+}
+
 void ModuleData::linkWith( const ModuleData* _module )
 {
   foreach( const llvm::Module* mod, _module->m_linkModuleWith )
@@ -213,6 +239,7 @@ void ModuleData::linkWith( const llvm::Module* _module )
 
 void ModuleData::doLink()
 {
+  std::list<GTLCore::String> hide;
   GTL_ASSERT( not m_llvmLinkedModule );
   m_llvmLinkedModule = m_llvmModule;
   m_llvmModule = llvm::CloneModule( m_llvmModule );
@@ -221,7 +248,31 @@ void ModuleData::doLink()
   foreach( const llvm::Module* mod, m_linkModuleWith )
   {
     llvm::Module* clone = llvm::CloneModule( mod );
+    
+    for(llvm::Module::iterator it = clone->getFunctionList().begin();
+        it != clone->getFunctionList().end(); ++it)
+    {
+      GTLCore::String name(it->getName());
+      if(name.isEmpty())
+      {
+        it->setLinkage(llvm::GlobalValue::InternalLinkage);
+      } else {
+        hide.push_back(name);
+      }
+    }
+    for(llvm::Module::global_iterator it = clone->getGlobalList().begin();
+        it != clone->getGlobalList().end(); ++it)
+    {
+      GTLCore::String name(it->getName());
+      if(name.isEmpty())
+      {
+        it->setLinkage(llvm::GlobalValue::InternalLinkage);
+      } else {
+        hide.push_back(name);
+      }
+    }
     linker.LinkInModule( clone, &errorMessage );
+    
     GTL_DEBUG("Linking error: " << errorMessage );
     delete clone;
   }
@@ -232,4 +283,34 @@ void ModuleData::doLink()
   }
   
   linker.releaseModule();
+  
+  // Hide symbols
+  foreach(GTLCore::String name, hide)
+  {
+    llvm::GlobalValue* value = m_llvmLinkedModule->getNamedValue((const std::string&)name);
+    if(value and not value->isDeclaration())
+    {
+      value->setLinkage(llvm::GlobalValue::InternalLinkage);
+    }
+  }
+}
+
+void ModuleData::hideAllSymbolsBut(std::list<GTLCore::String> _keepVisible)
+{
+  for(llvm::Module::iterator it = m_llvmLinkedModule->getFunctionList().begin();
+      it != m_llvmLinkedModule->getFunctionList().end(); ++it)
+  {
+    if(not it->isDeclaration() and std::find(_keepVisible.begin(), _keepVisible.end(), GTLCore::String(it->getName())) == _keepVisible.end() )
+    {
+      it->setLinkage(llvm::GlobalValue::InternalLinkage);
+    }
+  }
+  for(llvm::Module::global_iterator it = m_llvmLinkedModule->getGlobalList().begin();
+      it != m_llvmLinkedModule->getGlobalList().end(); ++it)
+  {
+    if(not it->isDeclaration() and std::find(_keepVisible.begin(), _keepVisible.end(), GTLCore::String(it->getName())) == _keepVisible.end() )
+    {
+      it->setLinkage(llvm::GlobalValue::InternalLinkage);
+    }
+  }
 }
