@@ -1,5 +1,5 @@
 /*
- *  Copyright (c) 2007,2010 Cyrille Berger <cberger@cberger.net>
+ *  Copyright (c) 2007,2008,2009,2010 Cyrille Berger <cberger@cberger.net>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -37,28 +37,34 @@ using namespace GTLCore;
 struct Type::StructDataMember::Private : public SharedPointerData
 {
   Private() {}
-  Private(const Private& _rhs) : name(_rhs.name), type(_rhs.type), initialSizes(_rhs.initialSizes) {}
+  Private(const Private& _rhs) : name(_rhs.name), type(_rhs.type), initialSizes(_rhs.initialSizes), information(new Information(*_rhs.information)) {}
+  ~Private() {
+    delete information;
+  }
   GTLCore::String name;
   const Type* type;
   std::list<int> initialSizes;
+  Information* information;
 };
 
 GTL_SHARED_DATA_CLASS(Type::StructDataMember, StructDataMember);
 
-Type::StructDataMember::StructDataMember(const GTLCore::String& _name, const Type* _type) : d(new Private)
+Type::StructDataMember::StructDataMember(const GTLCore::String& _name, const Type* _type, Information* information) : d(new Private)
 {
   d->ref();
   d->name = _name;
   d->type = _type;
+  d->information = information;
   GTL_ASSERT( _type->dataType() != Type::ARRAY);
 }
 
-Type::StructDataMember::StructDataMember(const GTLCore::String& _name, const Type* _type, const std::list<int>& _initialSizes) : d(new Private)
+Type::StructDataMember::StructDataMember(const GTLCore::String& _name, const Type* _type, const std::list<int>& _initialSizes, Information* _information) : d(new Private)
 {
   d->ref();
   d->name = _name;
   d->type = _type;
   d->initialSizes = _initialSizes;
+  d->information = _information;
   GTL_ASSERT( d->initialSizes.empty() or ( not d->initialSizes.empty() and _type->dataType() == Type::ARRAY) );
 }
 
@@ -77,6 +83,11 @@ const std::list<int>& Type::StructDataMember::initialSizes() const
   return d->initialSizes;
 }
 
+const Type::StructDataMember::Information* Type::StructDataMember::information() const
+{
+  return d->information;
+}
+
 //-------- Type -------//
 
 const Type* Type::Undefined = new Type(Type::UNDEFINED);
@@ -94,7 +105,7 @@ const Type* Type::Float32 = new Type(Type::FLOAT32);
 const Type* Type::Float16 = new Type(Type::FLOAT16);
 const Type* Type::Void = new Type(Type::VOID);
 const Type* Type::Pointer = new Type(Type::POINTER);
-const Type* Type::Color = new Type("Color", std::vector<Type::StructDataMember>());
+const Type* Type::Color = new Type("color", std::vector<Type::StructDataMember>());
 
 STATIC_DELETER(DeleteTypes)
 {
@@ -252,20 +263,45 @@ MatrixSize Type::matrixSize() const
 
 bool Type::isNumber() const
 {
-  return isInteger() or isFloatingPoint();
+  return (isInteger() or isFloatingPoint());
 }
 
 bool Type::isInteger() const
 {
-  return d->dataType == BOOLEAN or d->dataType == INTEGER8 or d->dataType == UNSIGNED_INTEGER8 or d->dataType == INTEGER16 or d->dataType == UNSIGNED_INTEGER16 or d->dataType == INTEGER32 or d->dataType == UNSIGNED_INTEGER32;
+  return d->dataType == BOOLEAN or d->dataType == INTEGER8 or d->dataType == UNSIGNED_INTEGER8 or d->dataType == INTEGER16 or d->dataType == UNSIGNED_INTEGER16 or d->dataType == INTEGER32 or d->dataType == UNSIGNED_INTEGER32 or ( d->dataType == VECTOR and d->arrayType->isInteger() );
 }
 
 bool Type::isFloatingPoint() const
 {
-  return d->dataType == FLOAT16 or d->dataType == FLOAT32 or d->dataType == FLOAT64;
+  return d->dataType == FLOAT16 or d->dataType == FLOAT32 or d->dataType == FLOAT64 or ( d->dataType == VECTOR and d->arrayType->isFloatingPoint() );
 }
 
 bool Type::isStructure() const
 {
   return d->dataType == STRUCTURE;
 }
+
+bool Type::isSameStructureLayout(const GTLCore::Type* type) const
+{
+  GTL_ASSERT(type->isStructure());
+  GTL_ASSERT(isStructure());
+  if (countStructDataMembers() != type->countStructDataMembers()) {
+    return false;
+  }
+  for(std::size_t i = 0; i < countStructDataMembers(); ++i)
+  {
+    Type::StructDataMember sm1 = structDataMember(i);
+    Type::StructDataMember sm2 = type->structDataMember(i);
+    
+    if( sm1.type() != sm2.type())
+    {
+      if( sm1.type()->dataType() != Type::STRUCTURE
+          or sm2.type()->dataType() != Type::STRUCTURE
+          or not sm1.type()->isSameStructureLayout(sm2.type()) )
+      {
+        return false;
+      }
+    }
+  }
+  return true;
+}
