@@ -1,5 +1,5 @@
 /*
- *  Copyright (c) 2008-2009 Cyrille Berger <cberger@cberger.net>
+ *  Copyright (c) 2008,2009,2010 Cyrille Berger <cberger@cberger.net>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -19,6 +19,8 @@
 
 #include "Type_p.h"
 
+#include <llvm/ADT/ArrayRef.h>
+
 #include "Debug.h"
 #include "Function.h"
 #include "Parameter.h"
@@ -31,6 +33,12 @@ using namespace GTLCore;
 #include <llvm/Type.h>
 #include <llvm/DerivedTypes.h>
 
+Type::StructDataMember::Information::~Information()
+{
+//   delete m_setter; // TODO fix me don't leak functions
+}
+
+
 struct Type::StructFunctionMember::Private
 {
   GTLCore::Function* function; // TODO make it shareable, and free memory
@@ -143,7 +151,7 @@ Type::Private::~Private()
   delete structPrivateFunctionMembers;
 }
 
-const llvm::Type * Type::Private::type(llvm::LLVMContext& _context) const
+llvm::Type * Type::Private::type(llvm::LLVMContext& _context) const
 {
   if(m_factory)
   {
@@ -180,7 +188,7 @@ const llvm::Type * Type::Private::type(llvm::LLVMContext& _context) const
     case Type::STRUCTURE:
     {
       // Init the llvm type
-      std::vector< const llvm::Type * > types;
+      std::vector< llvm::Type * > types;
       for(std::vector<StructDataMember>::iterator it = structDataMembers->begin();
           it != structDataMembers->end(); ++it)
       {
@@ -190,7 +198,7 @@ const llvm::Type * Type::Private::type(llvm::LLVMContext& _context) const
     }
     case Type::ARRAY:
     {
-      std::vector< const llvm::Type * > types;
+      std::vector< llvm::Type * > types;
       types.push_back( llvm::Type::getInt32Ty(_context)); // Counter from structure
       types.push_back( llvm::Type::getInt32Ty(_context));
       types.push_back( llvm::PointerType::get( arrayType->d->type(_context), 0));
@@ -206,12 +214,12 @@ const llvm::Type * Type::Private::type(llvm::LLVMContext& _context) const
   }
 }
 
-const llvm::Type * Type::Private::pointerType(llvm::LLVMContext& _context) const
+llvm::Type * Type::Private::pointerType(llvm::LLVMContext& _context) const
 {
   return llvm::PointerType::get( type(_context), 0);
 }
 
-const llvm::Type * Type::Private::asArgumentType(llvm::LLVMContext& _context) const
+llvm::Type * Type::Private::asArgumentType(llvm::LLVMContext& _context) const
 {
   if( dataType == Type::ARRAY or dataType == Type::STRUCTURE )
   {
@@ -250,18 +258,19 @@ int Type::Private::memberToIndex(const GTLCore::String& name)
   return -1;
 }
 
-const Type::StructFunctionMember* Type::Private::functionMember( const GTLCore::String& _name) const
+std::vector<const Type::StructFunctionMember*> Type::Private::functionMembers( const GTLCore::String& _name) const
 {
-  if(not structFunctionMembers) return 0;
+  std::vector<const Type::StructFunctionMember*> members;
+  if(not structFunctionMembers) return members;
   for( std::vector<GTLCore::Type::StructFunctionMember>::const_iterator it = structFunctionMembers->begin();
         it != structFunctionMembers->end(); ++it)
   {
     if( it->name() == _name)
     {
-      return &*it;
+      members.push_back(&*it);
     }
   }
-  return 0;
+  return members;
 }
 
 const Type::StructFunctionMember* Type::Private::privateFunctionMember( const GTLCore::String& _name) const
@@ -374,9 +383,9 @@ const GTLCore::Type* Type::Private::createArbitraryType( AribtraryTypeFactory* _
   return type;
 }
 
-std::vector<const llvm::Type*> Type::Private::createFunctionParams(llvm::LLVMContext& _context, const Function* function)
+std::vector< llvm::Type*> Type::Private::createFunctionParams(llvm::LLVMContext& _context, const Function* function)
 {
-  std::vector<const llvm::Type*> params;
+  std::vector< llvm::Type*> params;
   for(std::vector< Parameter >::const_iterator it = function->parameters().begin();
       it != function->parameters().end(); ++it)
   {
@@ -390,7 +399,7 @@ std::vector<const llvm::Type*> Type::Private::createFunctionParams(llvm::LLVMCon
   return params;
 }
 
-const llvm::FunctionType* Type::Private::createFunctionFunctionType(llvm::LLVMContext& _context, const Function* function)
+llvm::FunctionType* Type::Private::createFunctionFunctionType(llvm::LLVMContext& _context, const Function* function)
 {
   return llvm::FunctionType::get( function->returnType()->d->asArgumentType(_context), createFunctionParams(_context, function), false );
 }
