@@ -1,5 +1,5 @@
 /*
- *  Copyright (c) 2008 Cyrille Berger <cberger@cberger.net>
+ *  Copyright (c) 2008,2009,2010 Cyrille Berger <cberger@cberger.net>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -33,25 +33,28 @@
 #include "GTLCore/Type.h"
 #include "GTLCore/TypesManager.h"
 #include "GTLCore/Value.h"
+#include "GTLCore/Value_p.h"
 #include "GTLCore/VirtualMachine_p.h"
 #include "GTLCore/wrappers/Allocate.h"
 
+#include "GTLFragment/Library_p.h"
+
 #include "Debug.h"
-#include "Compiler_p.h"
 #include "Library_p.h"
 
 #include "Kernel_p.h"
 #include "Wrapper_p.h"
 #include "wrappers/ImageWrap_p.h"
 #include "wrappers/RegionWrap_p.h"
+#include <GTLCore/AST/Tree.h>
+#include <llvm/DerivedTypes.h>
 
 using namespace OpenShiva;
 
-Kernel::Kernel(int _channelsNb ) : Library( true, _channelsNb), d(new KernelPrivate )
+Kernel::Kernel(int _channelsNb ) : Library( GTLFragment::Library::SHIVA_KERNEL, _channelsNb), d(new KernelPrivate )
 {
   d->self = this;
   d->evaluatePixelsFunction = 0;
-  d->wrapper = 0;
   Library::d->libraryCompilation = d;
 }
 
@@ -64,21 +67,22 @@ Kernel::~Kernel()
 
 void Kernel::evaluatePixels( const GTLCore::RegionI& _region, const std::list< const GTLCore::AbstractImage* >& _inputImages, GTLCore::AbstractImage* _outputImage, GTLCore::ProgressReport* report, const GTLCore::Transform& transfo, const GTLCore::ChannelsFlags& flags) const
 {
+  d->runEvaluateDependentsIfNeeded();
   SHIVA_DEBUG( _region.x() << " " << _region.y() << " " << _region.columns() << " " << _region.rows());
   SHIVA_ASSERT( d->evaluatePixelsFunction );
   
   // Wrap input images
-  SHIVA_ASSERT( d->wrapper );
+  SHIVA_ASSERT( Library::d->m_wrapper );
   const void** inputImages = new const void*[ _inputImages.size() ];
   int i = 0;
   for( std::list< const GTLCore::AbstractImage* >::const_iterator it = _inputImages.begin();
        it != _inputImages.end(); ++it)
   {
-    inputImages[i] = (const void*)d->wrapper->wrapImage( const_cast<GTLCore::AbstractImage*>( *it ) ); // TODO no const_cast
+    inputImages[i] = (const void*)Library::d->m_wrapper->wrapImage( const_cast<GTLCore::AbstractImage*>( *it ) ); // TODO no const_cast
   }
   
   // Wrap output image
-  ImageWrap* owrap = d->wrapper->wrapImage( _outputImage );
+  ImageWrap* owrap = Library::d->m_wrapper->wrapImage( _outputImage );
   
   // Call function
   SHIVA_DEBUG("Call function");
@@ -98,8 +102,9 @@ void Kernel::evaluatePixels( const GTLCore::RegionI& _region, const std::list< c
 
 int Kernel::runTest() const
 {
+  d->runEvaluateDependentsIfNeeded();
   SHIVA_ASSERT( isCompiled() );
-  const std::list<GTLCore::Function*>* fs = Library::d->m_moduleData->function( name(), "runTest");
+  const std::list<GTLCore::Function*>* fs = GTLFragment::Library::d->m_moduleData->function( name(), "runTest");
   SHIVA_ASSERT( fs );
   GTLCore::Function* f = fs->front();
   GTLCore::Value v = f->call( std::vector< GTLCore::Value >() );
@@ -108,12 +113,13 @@ int Kernel::runTest() const
 
 bool Kernel::hasTestFunction() const
 {
-  return Library::d->m_moduleData->function( name(), "runTest");
+  return GTLFragment::Library::d->m_moduleData->function( name(), "runTest");
 }
 
 GTLCore::RegionF Kernel::needed( GTLCore::RegionI output_region, int input_index, const std::list< GTLCore::RegionI>& input_DOD, const GTLCore::Transform& transfo)
 {
-  const std::list<GTLCore::Function*>* neededFunctions = Library::d->m_moduleData->function( name(),"needed");
+  d->runEvaluateDependentsIfNeeded();
+  const std::list<GTLCore::Function*>* neededFunctions = GTLFragment::Library::d->m_moduleData->function( name(),"needed");
   GTL_ASSERT( neededFunctions );
   GTLCore::Function* neededFunction = neededFunctions->front();
   RegionWrap* (*func)( RegionWrap*, int, ArrayWrap* ) = ( RegionWrap* (*)( RegionWrap*, int, ArrayWrap* ) )GTLCore::VirtualMachine::instance()->getPointerToFunction( neededFunction);
@@ -126,12 +132,14 @@ GTLCore::RegionF Kernel::needed( GTLCore::RegionI output_region, int input_index
 
 bool Kernel::hasNeededFunction() const
 {
-  return Library::d->m_moduleData->function( name(), "needed");
+  return GTLFragment::Library::d->m_moduleData->function( name(), "needed");
 }
 
 GTLCore::RegionF Kernel::changed( GTLCore::RegionI changed_input_region, int input_index, const std::list< GTLCore::RegionI>& input_DOD, const GTLCore::Transform& transfo)
 {
-  const std::list<GTLCore::Function*>* changedFunctions = Library::d->m_moduleData->function( name(), "changed");
+  TEST_GUARD_READ(GTLFragment::Library::d)
+  d->runEvaluateDependentsIfNeeded();
+  const std::list<GTLCore::Function*>* changedFunctions = GTLFragment::Library::d->m_moduleData->function( name(), "changed");
   GTL_ASSERT(changedFunctions);
   GTLCore::Function* changedFunction = changedFunctions->front();
   RegionWrap* (*func)( RegionWrap*, int, ArrayWrap* ) = ( RegionWrap* (*)( RegionWrap*, int, ArrayWrap* ) )GTLCore::VirtualMachine::instance()->getPointerToFunction( changedFunction);
@@ -146,12 +154,15 @@ GTLCore::RegionF Kernel::changed( GTLCore::RegionI changed_input_region, int inp
 
 bool Kernel::hasChangedFunction() const
 {
-  return Library::d->m_moduleData->function( name(), "changed");
+  TEST_GUARD_READ(GTLFragment::Library::d)
+  return GTLFragment::Library::d->m_moduleData->function( name(), "changed");
 }
 
 GTLCore::RegionF Kernel::generated(const GTLCore::Transform& transfo)
 {
-  const std::list<GTLCore::Function*>*  fs = Library::d->m_moduleData->function( name(), "generated");
+  TEST_GUARD_READ(GTLFragment::Library::d)
+  d->runEvaluateDependentsIfNeeded();
+  const std::list<GTLCore::Function*>*  fs = GTLFragment::Library::d->m_moduleData->function( name(), "generated");
   SHIVA_ASSERT( fs );
   GTLCore::Function* f = fs->front();
 
@@ -165,7 +176,7 @@ GTLCore::RegionF Kernel::generated(const GTLCore::Transform& transfo)
 
 bool Kernel::hasGeneratedFunction() const
 {
-  return Library::d->m_moduleData->function( name(), "generated");
+  return GTLFragment::Library::d->m_moduleData->function( name(), "generated");
 }
 
 void Kernel::setParameter( StandardParameter _hint, const GTLCore::Value& _value)
@@ -187,10 +198,39 @@ void Kernel::setParameter( StandardParameter _hint, const GTLCore::Value& _value
   }
 }
 
+
 void Kernel::setParameter( const GTLCore::String& _name, const GTLCore::Value& _value )
 {
+  std::map< GTLCore::String, int >::iterator it = GTLFragment::Library::d->parameters_name2id.find(_name);
+  if(it == GTLFragment::Library::d->parameters_name2id.end())
+  {
+    setParameter(GTLFragment::Library::d->createParameter(_name, _value), _value);
+  } else {
+    setParameter(it->second, _value);
+  }
+}
+
+void Kernel::setParameter( std::size_t _id, const GTLCore::Value& _value)
+{
+  TEST_GUARD_WRITE(GTLFragment::Library::d)
   GTL_ASSERT(_value.isValid());
-  Library::d->parameters[ _name ] = _value;
+  GTLFragment::ParameterInfo& pi = GTLFragment::Library::d->parameters_info[ _id ];
+  pi.value = _value;
+  if(isCompiled())
+  {
+    GTLCore::String symbol = GTLCore::AST::GlobalConstantDeclaration::nameToSymbol(GTLCore::ScopedName(name(), pi.name));
+    llvm::GlobalVariable* variable = d->moduleData()->llvmLinkedModule()->getGlobalVariable((std::string)symbol);
+    GTL_ASSERT(variable);
+    void* ptr = GTLCore::VirtualMachine::instance()->getGlobalVariablePointer(variable);
+    GTL_ASSERT(ptr);
+    GTLCore::copyValueToPtr(_value, variable->getType()->getElementType(), d->moduleData()->llvmModule()->getContext(), ptr);
+    d->needRunEvealuateDependents = true;
+  }
+}
+
+std::size_t Kernel::parameterId( const GTLCore::String& _name )
+{
+  return GTLFragment::Library::d->parameters_name2id[_name];
 }
 
 void Kernel::setParameters( const std::map< GTLCore::String, GTLCore::Value>&  _values )
@@ -198,6 +238,6 @@ void Kernel::setParameters( const std::map< GTLCore::String, GTLCore::Value>&  _
   for( std::map< GTLCore::String, GTLCore::Value>::const_iterator it = _values.begin();
        it != _values.end(); ++it)
   {
-    Library::d->parameters[ it->first ] = it->second;
+    setParameter(it->first, it->second);
   }
 }
