@@ -28,6 +28,10 @@
 #include "GTLCore/TypesManager.h"
 #include "GTLCore/VirtualMachine_p.h"
 
+#include "GTLCore/AST/Tree.h"
+
+#include "GTLFragment/Library_p.h"
+
 #include "Debug.h"
 
 #include "CodeGenerator_p.h"
@@ -42,22 +46,21 @@ KernelPrivate::KernelPrivate()
 
 KernelPrivate::~KernelPrivate()
 {
-  delete wrapper;
 }
 
 void KernelPrivate::preCompilation()
 {
-  if (self->Library::d->parameters.find("IMAGE_WIDTH") == self->Library::d->parameters.end())
+  if (self->GTLFragment::Library::d->parameters_name2id.find("IMAGE_WIDTH") == self->GTLFragment::Library::d->parameters_name2id.end())
   {
     self->setParameter( Kernel::IMAGE_WIDTH, 800.0f );
   }
-  if (self->Library::d->parameters.find("IMAGE_HEIGHT") == self->Library::d->parameters.end())
+  if (self->GTLFragment::Library::d->parameters_name2id.find("IMAGE_HEIGHT") == self->GTLFragment::Library::d->parameters_name2id.end())
   {
     self->setParameter( Kernel::IMAGE_HEIGHT, 600.0f );
   }
   std::vector<GTLCore::Value> values;
-  values.push_back( self->Library::d->parameters["IMAGE_WIDTH"]);
-  values.push_back( self->Library::d->parameters["IMAGE_HEIGHT"]);
+  values.push_back( self->GTLFragment::Library::d->parameters_info[ self->parameterId("IMAGE_WIDTH")].value);
+  values.push_back( self->GTLFragment::Library::d->parameters_info[ self->parameterId("IMAGE_HEIGHT")].value);
   self->setParameter( "IMAGE_SIZE", GTLCore::Value(values, GTLCore::TypesManager::getVector(GTLCore::Type::Float32, 2) ) );
 }
 
@@ -68,13 +71,37 @@ void KernelPrivate::cleanup()
 
 void KernelPrivate::postCompilation()
 {
+  std::list<GTLCore::String> keepVisible;
+  
+  // Functions
+  keepVisible.push_back("evaluateDependents");
+  keepVisible.push_back("evaluatePixels");
+  keepVisible.push_back("runTest");
+  keepVisible.push_back("changed");
+  keepVisible.push_back("needed");
+  keepVisible.push_back("generated");
+  
+  // Parmeters
+  foreach(const GTLFragment::ParameterInfo& pi, self->GTLFragment::Library::d->parameters_info)
+  {
+    GTLCore::String symbol = GTLCore::AST::GlobalConstantDeclaration::nameToSymbol(GTLCore::ScopedName(self->name(), pi.name));
+    keepVisible.push_back(symbol);
+  }
+  self->GTLFragment::Library::d->m_moduleData->hideAllSymbolsBut(keepVisible);
+  
   // Create a wrapper
-  wrapper = new Wrapper(self, self->Library::d->m_moduleData, self->Library::d->m_llvmModule);
   determineTypes();
   // Create the generateEvaluatePixels LLVM function
-  evaluatePixelsFunction = CodeGenerator::generateEvaluatePixels( self, self->Library::d->m_llvmModule, self->Library::d->count_channels_generic );
+  evaluatePixelsFunction = CodeGenerator::generateEvaluatePixels( self, self->GTLFragment::Library::d->m_moduleData->llvmLinkedModule(), self->GTLFragment::Library::d->count_channels_generic );
+  needRunEvealuateDependents = true;
+  runEvaluateDependentsIfNeeded();
+}
+
+void KernelPrivate::runEvaluateDependentsIfNeeded()
+{
+  if(not needRunEvealuateDependents) return;
   // Call evaluateDepends as needed
-  const std::list<GTLCore::Function*>* evaluateDependentsFuncs = self->Library::d->m_moduleData->function( self->name(), "evaluateDependents");
+  const std::list<GTLCore::Function*>* evaluateDependentsFuncs = self->GTLFragment::Library::d->m_moduleData->function( self->name(), "evaluateDependents");
   if( evaluateDependentsFuncs )
   {
     GTLCore::Function* evaluateDependentsFunc = evaluateDependentsFuncs->front();
@@ -85,13 +112,13 @@ void KernelPrivate::postCompilation()
 
 GTLCore::ModuleData* KernelPrivate::moduleData()
 {
-  return self->Library::d->m_moduleData;
+  return self->GTLFragment::Library::d->m_moduleData;
 }
 
 void KernelPrivate::determineTypes()
 {
   m_inputsTypes.clear();
-  const std::list<GTLCore::Function*>*  ePFunctions = self->Library::d->m_moduleData->function( self->name(), "evaluatePixel" );
+  const std::list<GTLCore::Function*>*  ePFunctions = self->GTLFragment::Library::d->m_moduleData->function( self->name(), "evaluatePixel" );
   GTL_ASSERT(ePFunctions);
   GTLCore::Function* ePFunction = ePFunctions->front();
   bool hasOutput = false;
@@ -106,9 +133,9 @@ void KernelPrivate::determineTypes()
       SHIVA_ASSERT( m_outputPixelType->structName().head( 5 ) == "pixel" );
       if(  m_outputPixelType->structName() == "pixel" )
       {
-        m_outputImageType = self->Library::d->m_moduleData->typesManager()->getStructure( "image" );
+        m_outputImageType = self->GTLFragment::Library::d->m_moduleData->typesManager()->getStructure( "image" );
       } else {
-        m_outputImageType = self->Library::d->m_moduleData->typesManager()->getStructure( "image" + m_outputPixelType->structName().tail(1) );
+        m_outputImageType = self->GTLFragment::Library::d->m_moduleData->typesManager()->getStructure( "image" + m_outputPixelType->structName().tail(1) );
       }
       GTL_ASSERT( m_outputImageType );
     } else {
@@ -134,3 +161,18 @@ const GTLCore::Type* KernelPrivate::outputImageType()
 {
   return m_outputImageType;
 }
+
+void KernelPrivate::createWrapper(GTLCore::ModuleData* _moduleData, llvm::Module* module, int _channels, GTLFragment::WhichStdLibrary _library)
+{
+  self->Library::d->m_wrapper = new Wrapper(self, _moduleData, module, _channels, _library != GTLFragment::ShivaWrappersLibrary);
+}
+
+GTLFragment::Wrapper* KernelPrivate::wrapper()
+{
+  GTL_ABORT("should not happen");
+}
+
+void KernelPrivate::addOperatorOverloads(GTLCore::OperatorOverloadCenter* _center)
+{
+  GTL_ABORT("should not happen");
+}
