@@ -20,13 +20,8 @@
 #ifndef _OPENSHIVA_KERNEL_SOURCE_H_
 #define _OPENSHIVA_KERNEL_SOURCE_H_
 
-#include <list>
 #include <OpenShiva/Export.h>
-
-namespace GTLCore {
-  class CompilationMessages;
-  class String;
-}
+#include <GTLFragment/Source.h>
 
 namespace OpenShiva {
   class Metadata;
@@ -36,21 +31,12 @@ namespace OpenShiva {
    * Define the source code of a Kernel. Allow to access to information like the metadata,
    * or the source code, or the parameters of the evaluateParam function.
    */
-  class OPENSHIVA_EXPORT Source {
+  class OPENSHIVA_EXPORT Source : public GTLFragment::Source {
     public:
       Source();
       Source(const Source& );
       Source& operator=(const Source& );
       ~Source();
-      GTLCore::String name() const;
-      GTLCore::String source() const;
-      void setSource( const GTLCore::String& );
-      void loadFromFile(const GTLCore::String& _fileName);
-      /**
-       * @return the metadata for this kernel.
-       */
-      const Metadata* metadata() const;
-      GTLCore::CompilationMessages metadataCompilationMessages() const;
     public:
       enum SourceType {
         InvalidSource, ///< Not a valid Shiva source code
@@ -84,11 +70,9 @@ namespace OpenShiva {
        * @return the number of input images (-1 indicates the input functions takes an array)
        */
       int countInputImages() const;
-    private:
-      inline void deref();
     private:
       struct Private;
-      Private* d;
+      inline const Private* d_func() const;
   };
 }
 
