@@ -46,6 +46,8 @@
 #include "PixelConvertExpressionFactory_p.h"
 
 #include "wrappers/ImageWrap_p.h"
+#include "LibrariesManager.h"
+#include <GTLCore/CompilationMessages.h>
 
 using namespace OpenShiva;
 
@@ -61,13 +63,14 @@ typedef pdToWF::const_iterator pdToWF_cit;
 struct Wrapper::Private {
   pdToWF imageFunctions;
   Kernel* kernel;
-  GTLCore::ModuleData* moduleData;
+  bool loadShivaWrapper;
 };
 
-Wrapper::Wrapper(Kernel* _kernel, GTLCore::ModuleData* _moduleData) : d(new Private)
+Wrapper::Wrapper(Kernel* _kernel, GTLCore::ModuleData* _moduleData, llvm::Module* module, int channels, bool loadShivaWrapper) : GTLFragment::Wrapper(_moduleData, module, channels), d(new Private)
 {
+  d->loadShivaWrapper = loadShivaWrapper;
+  GTL_ASSERT( channels > 0 );
   d->kernel = _kernel;
-  d->moduleData = _moduleData;
 }
 
 Wrapper::~Wrapper()
@@ -86,14 +89,14 @@ ImageWrap* Wrapper::wrapImage(GTLCore::AbstractImage* _abstractImage)
     // Wrap functions doesn't exist yet, generate them
     WrappedFunctions wf;
     wf.memToVec = GTLCore::VirtualMachine::instance()->getPointerToFunction(
-        CodeGenerator::generateMemToVec( d->moduleData, _abstractImage->pixelDescription()  ) );
+        CodeGenerator::generateMemToVec( moduleData(), llvmModule(), _abstractImage->pixelDescription()  ) );
     wf.vecToMem = GTLCore::VirtualMachine::instance()->getPointerToFunction(
-        CodeGenerator::generateVecToMem( d->moduleData, _abstractImage->pixelDescription()  ) );
+        CodeGenerator::generateVecToMem( moduleData(), llvmModule(), _abstractImage->pixelDescription()  ) );
     d->imageFunctions[ _abstractImage->pixelDescription() ] = wf;
     owrap->memToVec = wf.memToVec;
     owrap->vecToMem = wf.vecToMem;
   } else {
-    // Use allready existing wrap function
+    // Use already existing wrap function
     owrap->memToVec = it->second.memToVec;
     owrap->vecToMem = it->second.vecToMem;
   }
@@ -101,26 +104,44 @@ ImageWrap* Wrapper::wrapImage(GTLCore::AbstractImage* _abstractImage)
   return owrap;
 }
 
-void Wrapper::fillTypesManager( GTLCore::ModuleData* _module, GTLCore::TypesManager* _typesManager, GTLCore::ConvertCenter* _convertCenter, int _channels )
+void Wrapper::fillTypesManager( GTLCore::TypesManager* _typesManager, GTLCore::ConvertCenter* _convertCenter )
 {
-  GTL_ASSERT(_channels > 0 );
   SHIVA_DEBUG("fillTypesManager");
+  /*const GTLCore::Type* colorType = */createColorType(_typesManager, _convertCenter);
   _convertCenter->addConvertExpressionFactory(new PixelConvertExpressionFactory);
   for( int i = 1; i <= 5; ++i)
   {
     // Create Image type
-    const GTLCore::Type* pixelType = createPixelType( _module, _typesManager, _convertCenter, GTLCore::String::number( i ), i );
-    createImageType( _module, _typesManager, GTLCore::String::number( i ), i, pixelType );
+    const GTLCore::Type* pixelType = createPixelType( moduleData(), llvmModule(), _typesManager, _convertCenter, GTLCore::String::number( i ), i );
+    createImageType( moduleData(), llvmModule(), _typesManager, GTLCore::String::number( i ), i, pixelType );
+  }
+  const GTLCore::Type* pixelType = createPixelType( moduleData(), llvmModule(), _typesManager, _convertCenter, "", channels() );
+  createImageType( moduleData(), llvmModule(), _typesManager, "", channels(), pixelType );
+  const GTLCore::Type* regionType = createRegionType( moduleData(), llvmModule(), _typesManager );
+
+  if(d->loadShivaWrapper)
+  {
+    GTLFragment::Library* library = OpenShiva::LibrariesManager::instance()->loadLibrary( "shivawrappers", channels());
+    GTL_ASSERT( library );
+    if(not library->isCompiled())
+    {
+      library->compile();
+      if(not library->isCompiled())
+      {
+        GTL_ABORT("shivawrappers.shiva compilation failed, " << library->compilationMessages().toString());
+        return;
+      }
+    }
+    moduleData()->linkWith( library->data() );
+    addFunctionFromModuleToType(regionType, library, GTLCore::ScopedName("wrappers", "translate"));
+    addFunctionFromModuleToType(regionType, library, GTLCore::ScopedName("wrappers", "translated"));
   }
-  const GTLCore::Type* pixelType = createPixelType( _module, _typesManager, _convertCenter, "", _channels );
-  createImageType( _module, _typesManager, "", _channels, pixelType );
-  createRegionType( _module, _typesManager );
 }
 
 
 llvm::FunctionType* Wrapper::pixel_wrap_set_alpha_type(llvm::LLVMContext& context, GTLCore::TypesManager* _typesManager, const GTLCore::Type* _pixelType)
 {
-  std::vector<const llvm::Type*> functionArgs;
+  std::vector<llvm::Type*> functionArgs;
   functionArgs.push_back( _pixelType->d->pointerType(context) );
   functionArgs.push_back( llvm::Type::getFloatTy(context) );
   return llvm::FunctionType::get( llvm::Type::getVoidTy(context), functionArgs, false);
@@ -128,16 +149,16 @@ llvm::FunctionType* Wrapper::pixel_wrap_set_alpha_type(llvm::LLVMContext& contex
 
 llvm::FunctionType* Wrapper::pixel_wrap_alpha_type( llvm::LLVMContext& context, GTLCore::TypesManager* _typesManager, const GTLCore::Type* _pixelType)
 {
-  std::vector<const llvm::Type*> functionArgs;
+  std::vector<llvm::Type*> functionArgs;
   functionArgs.push_back( _pixelType->d->pointerType(context) );
   return llvm::FunctionType::get( llvm::Type::getFloatTy(context), functionArgs, false);
 }
 
-const GTLCore::Type* Wrapper::createPixelType( GTLCore::ModuleData* _module, GTLCore::TypesManager* _typesManager, GTLCore::ConvertCenter* _convertCenter, const GTLCore::String& _suffix, int _channels )
+const GTLCore::Type* Wrapper::createPixelType( GTLCore::ModuleData* _module, llvm::Module* llvmModule, GTLCore::TypesManager* _typesManager, GTLCore::ConvertCenter* _convertCenter, const GTLCore::String& _suffix, int _channels )
 {
   //---------------------- WARNING ----------------------//
   // Whenever the following structure is edited,         //
-  // it's llvm declaration must be changed too in        //
+  // its C++ declaration must be changed too in          //
   // struct PixelWrap !                                  //
   //---------------------- WARNING ----------------------//
   
@@ -145,27 +166,28 @@ const GTLCore::Type* Wrapper::createPixelType( GTLCore::ModuleData* _module, GTL
   
   const GTLCore::Type* vecType = vectorType( _typesManager, _channels );
   std::vector<GTLCore::Type::StructDataMember> pixelDataMembers;
+  pixelDataMembers.push_back( GTLCore::Type::StructDataMember( "", GTLCore::Type::Pointer ) ); // AbstractColorConverter*
+  pixelDataMembers.push_back( GTLCore::Type::StructDataMember( "", GTLCore::Type::Integer32 ) ); // alphapos
+  
+  pixelDataMembers.push_back( GTLCore::Type::StructDataMember( "coord", _typesManager->getVector( GTLCore::Type::Float32, 2 ) ) );
+  
   pixelDataMembers.push_back( GTLCore::Type::StructDataMember( "data", vecType ) );
   
-  pixelDataMembers.push_back( GTLCore::Type::StructDataMember( "coord", _typesManager->getVector( GTLCore::Type::Float, 2 ) ) );
-  pixelDataMembers.push_back( GTLCore::Type::StructDataMember( "", GTLCore::Type::Integer32 ) );
-  
-  
   
   const GTLCore::Type* type = _typesManager->d->createStructure( "pixel" + _suffix, pixelDataMembers );
   type->d->setVisitor( PixelVisitor::instance() );
   
   type->d->addFunctionMember( GTLCore::Type::StructFunctionMember(
           GTLCore::Function::Private::createInternalFunction(
-                _module, context, "setAlpha",
-                CodeGenerator::generatePixelSetAlpha( _module, type, _channels ),
-                GTLCore::Type::Void, 2,
-                type, GTLCore::Type::Float ) ) );
+                _module, context, "setOpacity",
+                CodeGenerator::generatePixelSetAlpha( _module, llvmModule, type, _channels ),
+                GTLCore::Type::Void, GTLCore::Function::Private::EFP_ONLY_TYPE, 2,
+                type, GTLCore::Type::Float32 ) ) );
   type->d->addFunctionMember( GTLCore::Type::StructFunctionMember(
           GTLCore::Function::Private::createInternalFunction(
-                _module, context, "alpha",
-                CodeGenerator::generatePixelAlpha( _module, type, _channels ),
-                GTLCore::Type::Float, 1,
+                _module, context, "opacity",
+                CodeGenerator::generatePixelAlpha( _module, llvmModule, type, _channels ),
+                GTLCore::Type::Float32, GTLCore::Function::Private::EFP_ONLY_TYPE, 1,
                 type ) ) );
 
   _convertCenter->addAutoConversion( type, vecType );
@@ -173,54 +195,63 @@ const GTLCore::Type* Wrapper::createPixelType( GTLCore::ModuleData* _module, GTL
   return type;
 }
 
-void Wrapper::createRegionType( GTLCore::ModuleData* _module, GTLCore::TypesManager* _typesManager )
+const GTLCore::Type* Wrapper::createRegionType( GTLCore::ModuleData* _module, llvm::Module* _llvmModule, GTLCore::TypesManager* _typesManager )
 {
   //---------------------- WARNING ----------------------//
   // Whenever the following structure is edited,         //
-  // it's llvm declaration must be changed too in        //
+  // its C++ declaration must be changed too in          //
   // struct RegionWrap !                                 //
   //---------------------- WARNING ----------------------//
   
   llvm::LLVMContext& context = _module->llvmModule()->getContext();
   
   std::vector<GTLCore::Type::StructDataMember> regionDataMembers;
-  regionDataMembers.push_back( GTLCore::Type::StructDataMember( "x", GTLCore::Type::Float ) );
-  regionDataMembers.push_back( GTLCore::Type::StructDataMember( "y", GTLCore::Type::Float) );
-  regionDataMembers.push_back( GTLCore::Type::StructDataMember( "width", GTLCore::Type::Float ) );
-  regionDataMembers.push_back( GTLCore::Type::StructDataMember( "height", GTLCore::Type::Float ) );
+  regionDataMembers.push_back( GTLCore::Type::StructDataMember( "x", GTLCore::Type::Float32 ) );
+  regionDataMembers.push_back( GTLCore::Type::StructDataMember( "y", GTLCore::Type::Float32) );
+  regionDataMembers.push_back( GTLCore::Type::StructDataMember( "columns", GTLCore::Type::Float32 ) );
+  regionDataMembers.push_back( GTLCore::Type::StructDataMember( "rows", GTLCore::Type::Float32 ) );
   const GTLCore::Type* type = _typesManager->d->createStructure( "region", regionDataMembers);
   type->d->addFunctionMember( GTLCore::Type::StructFunctionMember(
           GTLCore::Function::Private::createExternalFunction(
-                _module, context, "left", "region_wrap_left", GTLCore::Type::Float, 1,
+                _module, _llvmModule, context, "left", "region_wrap_left", GTLCore::Type::Float32,
+                GTLCore::Function::Private::EFP_ONLY_TYPE, 1,
                 type ) ) );
   type->d->addFunctionMember( GTLCore::Type::StructFunctionMember(
           GTLCore::Function::Private::createExternalFunction(
-                _module, context, "right", "region_wrap_right", GTLCore::Type::Float, 1,
+                _module, _llvmModule, context, "right", "region_wrap_right", GTLCore::Type::Float32,
+                GTLCore::Function::Private::EFP_ONLY_TYPE, 1,
                 type ) ) );
   type->d->addFunctionMember( GTLCore::Type::StructFunctionMember(
           GTLCore::Function::Private::createExternalFunction(
-                _module, context, "bottom", "region_wrap_bottom", GTLCore::Type::Float, 1,
+                _module, _llvmModule, context, "bottom", "region_wrap_bottom", GTLCore::Type::Float32,
+                GTLCore::Function::Private::EFP_ONLY_TYPE, 1,
                 type ) ) );
   type->d->addFunctionMember( GTLCore::Type::StructFunctionMember(
           GTLCore::Function::Private::createExternalFunction(
-                _module, context, "top", "region_wrap_top", GTLCore::Type::Float, 1,
+                _module, _llvmModule, context, "top", "region_wrap_top", GTLCore::Type::Float32,
+                GTLCore::Function::Private::EFP_ONLY_TYPE, 1,
                 type ) ) );
   type->d->addFunctionMember( GTLCore::Type::StructFunctionMember(
           GTLCore::Function::Private::createExternalFunction(
-                _module, context, "intersect", "region_wrap_intersect", GTLCore::Type::Void, 2,
+                _module, _llvmModule, context, "intersect", "region_wrap_intersect", GTLCore::Type::Void,
+                GTLCore::Function::Private::EFP_ONLY_TYPE, 2,
                 type, type ) ) );
   type->d->addFunctionMember( GTLCore::Type::StructFunctionMember(
           GTLCore::Function::Private::createExternalFunction(
-                _module, context, "union", "region_wrap_union", GTLCore::Type::Void, 2,
+                _module, _llvmModule, context, "union", "region_wrap_union", GTLCore::Type::Void,
+                GTLCore::Function::Private::EFP_ONLY_TYPE, 2,
                 type, type ) ) );
   type->d->addFunctionMember( GTLCore::Type::StructFunctionMember(
           GTLCore::Function::Private::createExternalFunction(
-                _module, context, "outset", "region_wrap_outset", GTLCore::Type::Void, 2,
-                type, GTLCore::Type::Float ) ) );
+                _module, _llvmModule, context, "outset", "region_wrap_outset", GTLCore::Type::Void,
+                GTLCore::Function::Private::EFP_ONLY_TYPE, 2,
+                type, GTLCore::Type::Float32 ) ) );
   type->d->addFunctionMember( GTLCore::Type::StructFunctionMember(
           GTLCore::Function::Private::createExternalFunction(
-                _module, context, "inset", "region_wrap_inset", GTLCore::Type::Void, 2,
-                type, GTLCore::Type::Float ) ) );
+                _module, _llvmModule, context, "inset", "region_wrap_inset", GTLCore::Type::Void,
+                GTLCore::Function::Private::EFP_ONLY_TYPE, 2,
+                type, GTLCore::Type::Float32 ) ) );
+  return type;
 }
 
 llvm::Function* Wrapper::image_wrap_dataFunction( llvm::Module* _module, const GTLCore::Type* _imageType )
@@ -229,7 +260,7 @@ llvm::Function* Wrapper::image_wrap_dataFunction( llvm::Module* _module, const G
   
   llvm::LLVMContext& context = _module->getContext();
   
-  std::vector<const llvm::Type*> functionArgs;
+  std::vector<llvm::Type*> functionArgs;
   functionArgs.push_back(llvm::PointerType::get( _imageType->d->type(context), 0));
   functionArgs.push_back(llvm::Type::getInt32Ty(context));
   functionArgs.push_back(llvm::Type::getInt32Ty(context));
@@ -240,7 +271,7 @@ llvm::Function* Wrapper::image_wrap_dataFunction( llvm::Module* _module, const G
 llvm::Function* Wrapper::image_wrap_const_dataFunction( llvm::Module* _module, const GTLCore::Type* _imageType )
 {
   GTL_ASSERT( _imageType );
-  std::vector<const llvm::Type*> functionArgs;
+  std::vector<llvm::Type*> functionArgs;
   functionArgs.push_back(llvm::PointerType::get( _imageType->d->type(_module->getContext()), 0));
   functionArgs.push_back(llvm::IntegerType::get(_module->getContext(), 32));
   functionArgs.push_back(llvm::IntegerType::get(_module->getContext(), 32));
@@ -255,23 +286,35 @@ llvm::Function* Wrapper::image_alpha_posFunction( llvm::Module* _module, const G
   
   llvm::LLVMContext& context = _module->getContext();
   
-  std::vector<const llvm::Type*> functionArgs;
+  std::vector<llvm::Type*> functionArgs;
   functionArgs.push_back(llvm::PointerType::get( _imageType->d->type(context), 0));
   llvm::FunctionType* functionTy = llvm::FunctionType::get(llvm::IntegerType::getInt32Ty(context), functionArgs, false);
   return (llvm::Function*)_module->getOrInsertFunction( "image_alpha_pos", functionTy);
 }
 
+llvm::Function* Wrapper::image_color_converter( llvm::Module* _module, const GTLCore::Type* _imageType )
+{
+  GTL_ASSERT( _imageType );
+  
+  llvm::LLVMContext& context = _module->getContext();
+  
+  std::vector<llvm::Type*> functionArgs;
+  functionArgs.push_back(llvm::PointerType::get( _imageType->d->type(context), 0));
+  llvm::FunctionType* functionTy = llvm::FunctionType::get(GTLCore::Type::Pointer->d->type(_module->getContext()), functionArgs, false);
+  return (llvm::Function*)_module->getOrInsertFunction( "image_color_converter", functionTy);
+}
+
 llvm::FunctionType* Wrapper::image_wrap_sample_nearest_type( llvm::LLVMContext& context, GTLCore::TypesManager* _typesManager, const GTLCore::Type* _imageType, const GTLCore::Type* _pixelType )
 {
-  std::vector<const llvm::Type*> functionArgs;
+  std::vector<llvm::Type*> functionArgs;
   functionArgs.push_back( _imageType->d->pointerType(context) );
-  functionArgs.push_back( _typesManager->getVector( GTLCore::Type::Float, 2 )->d->type(context) );
+  functionArgs.push_back( _typesManager->getVector( GTLCore::Type::Float32, 2 )->d->type(context) );
   return llvm::FunctionType::get( _pixelType->d->pointerType(context) , functionArgs, false);
 }
 
 llvm::FunctionType* Wrapper::image_wrap_mem_to_vec_float_type( llvm::LLVMContext& context, GTLCore::TypesManager* _typesManager, int _channels )
 {
-  std::vector<const llvm::Type*> functionArgs;
+  std::vector<llvm::Type*> functionArgs;
   functionArgs.push_back( llvm::PointerType::get( vectorType( _typesManager, _channels)->d->type(context), 0) );
   functionArgs.push_back( llvm::PointerType::get( llvm::Type::getInt8Ty(context), 0));
   return llvm::FunctionType::get( llvm::Type::getVoidTy(context), functionArgs, false);
@@ -279,9 +322,10 @@ llvm::FunctionType* Wrapper::image_wrap_mem_to_vec_float_type( llvm::LLVMContext
 
 llvm::FunctionType* Wrapper::image_wrap_vec_float_to_mem_type( llvm::LLVMContext& context, GTLCore::TypesManager* _typesManager, int _channels )
 {
-  std::vector<const llvm::Type*> functionArgs;
+  std::vector<llvm::Type*> functionArgs;
   functionArgs.push_back( llvm::PointerType::get( llvm::IntegerType::getInt8Ty(context), 0));
   functionArgs.push_back( llvm::PointerType::get( vectorType( _typesManager, _channels)->d->type(context), 0) );
+  functionArgs.push_back( llvm::Type::getInt64Ty(context) ); // gtl_uint64 _channelFlags
   return llvm::FunctionType::get( llvm::Type::getVoidTy(context), functionArgs, false);
 }
 
@@ -315,11 +359,11 @@ class Wrapper::VecFloat2MemTypeAribtraryTypeFactory : public GTLCore::Type::Priv
     llvm::LLVMContext& m_context;
 };
 
-void Wrapper::createImageType( GTLCore::ModuleData* _moduleData, GTLCore::TypesManager* _typesManager, const GTLCore::String& _suffix, int _channels, const GTLCore::Type* _pixelType )
+void Wrapper::createImageType( GTLCore::ModuleData* _moduleData, llvm::Module* _llvmModule, GTLCore::TypesManager* _typesManager, const GTLCore::String& _suffix, int _channels, const GTLCore::Type* _pixelType )
 {
   //---------------------- WARNING ----------------------//
   // Whenever the following structure is edited,         //
-  // it's llvm declaration must be changed too in        //
+  // its C++ declaration must be changed too in          //
   // struct ImageWrap !                                  //
   //---------------------- WARNING ----------------------//
   std::vector<GTLCore::Type::StructDataMember> imageDataMembers;
@@ -335,16 +379,16 @@ void Wrapper::createImageType( GTLCore::ModuleData* _moduleData, GTLCore::TypesM
   type->d->addFunctionMember( GTLCore::Type::StructFunctionMember(
           GTLCore::Function::Private::createInternalFunction(
                 _moduleData, _moduleData->llvmModule()->getContext(), "sampleNearest",
-                CodeGenerator::generateImageSampleNearest( _moduleData, type, _pixelType ),
-                _pixelType, 2,
-                type, _moduleData->typesManager()->getVector( GTLCore::Type::Float, 2 ) ) ) );
+                CodeGenerator::generateImageSampleNearest( _moduleData, _llvmModule, type, _pixelType ),
+                _pixelType, GTLCore::Function::Private::EFP_ONLY_TYPE, 2,
+                type, _moduleData->typesManager()->getVector( GTLCore::Type::Float32, 2 ) ) ) );
 }
 
 const GTLCore::Type* Wrapper::vectorType( GTLCore::TypesManager* _typesManager, int _channels )
 {
   if( _channels == 1)
   {
-    return GTLCore::Type::Float;
+    return GTLCore::Type::Float32;
   }
-  return _typesManager->getVector( GTLCore::Type::Float, _channels);
+  return _typesManager->getVector( GTLCore::Type::Float32, _channels);
 }
