@@ -19,6 +19,7 @@
 
 #include "GTLTest/Thread.h"
 #include "GTLCore/Image.h"
+#include "GTLCore/Type.h"
 
 #ifdef GTLTEST_HAVE_THREAD
 
@@ -75,6 +76,7 @@ class TestKernelThreading : public GTLTest::Case {
       
       MyThreadTCTLKT thread1(&k, &img, &img2, GTLCore::RegionI(0,0,50,50));
       thread1.start();
+      sleep(1);
       MyThreadTCTLKT thread2(&k, &img, &img2, GTLCore::RegionI(0,50,50,50));
       thread2.start();
       MyThreadTCTLKT thread3(&k, &img, &img2, GTLCore::RegionI(50,0,50,50));
